import unittest
from unittest import mock

from django.db import models
from django.test import TestCase, override_settings
from django.utils.datastructures import MultiValueDict

from django_filters.exceptions import FieldLookupError
from django_filters.filters import (
    BaseInFilter,
    BaseRangeFilter,
    BooleanFilter,
    CharFilter,
    ChoiceFilter,
    DateRangeFilter,
    Filter,
    FilterMethod,
    ModelChoiceFilter,
    ModelMultipleChoiceFilter,
    NumberFilter,
    UUIDFilter,
)
from django_filters.filterset import (
    FILTER_FOR_DBFIELD_DEFAULTS,
    FilterSet,
    filterset_factory,
)
from django_filters.widgets import BooleanWidget

from .models import (
    Account,
    AdminUser,
    Article,
    BankAccount,
    Book,
    Business,
    Comment,
    DirectedNode,
    NetworkSetting,
    Node,
    Profile,
    Restaurant,
    SubnetMaskField,
    User,
    UUIDTestModel,
    Worker,
)
from .utils import MockQuerySet


class HelperMethodsTests(TestCase):
    @unittest.skip("todo")
    def test_get_declared_filters(self):
        pass

    @unittest.skip("todo")
    def test_filters_for_model(self):
        pass


class DbFieldDefaultFiltersTests(TestCase):
    def test_expected_db_fields_get_filters(self):
        to_check = [
            models.BooleanField,
            models.CharField,
            models.CommaSeparatedIntegerField,
            models.DateField,
            models.DateTimeField,
            models.DecimalField,
            models.EmailField,
            models.FilePathField,
            models.FloatField,
            models.IntegerField,
            models.GenericIPAddressField,
            models.NullBooleanField,
            models.PositiveIntegerField,
            models.PositiveSmallIntegerField,
            models.SlugField,
            models.SmallIntegerField,
            models.TextField,
            models.TimeField,
            models.DurationField,
            models.URLField,
            models.ForeignKey,
            models.OneToOneField,
            models.ManyToManyField,
            models.UUIDField,
        ]
        msg = "%s expected to be found in FILTER_FOR_DBFIELD_DEFAULTS"

        for m in to_check:
            self.assertIn(m, FILTER_FOR_DBFIELD_DEFAULTS, msg % m.__name__)

    def test_expected_db_fields_do_not_get_filters(self):
        to_check = [
            models.Field,
            models.BigIntegerField,
            models.FileField,
            models.ImageField,
        ]
        msg = "%s expected to not be found in FILTER_FOR_DBFIELD_DEFAULTS"

        for m in to_check:
            self.assertNotIn(m, FILTER_FOR_DBFIELD_DEFAULTS, msg % m.__name__)


class FilterSetFilterForFieldTests(TestCase):
    def test_filter_found_for_field(self):
        f = User._meta.get_field("username")
        result = FilterSet.filter_for_field(f, "username")
        self.assertIsInstance(result, CharFilter)
        self.assertEqual(result.field_name, "username")

    def test_filter_found_for_uuidfield(self):
        f = UUIDTestModel._meta.get_field("uuid")
        result = FilterSet.filter_for_field(f, "uuid")
        self.assertIsInstance(result, UUIDFilter)
        self.assertEqual(result.field_name, "uuid")

    def test_filter_found_for_autofield(self):
        f = User._meta.get_field("id")
        result = FilterSet.filter_for_field(f, "id")
        self.assertIsInstance(result, NumberFilter)
        self.assertEqual(result.field_name, "id")

    def test_field_with_extras(self):
        f = User._meta.get_field("favorite_books")
        result = FilterSet.filter_for_field(f, "favorite_books")
        self.assertIsInstance(result, ModelMultipleChoiceFilter)
        self.assertEqual(result.field_name, "favorite_books")
        self.assertTrue("queryset" in result.extra)
        self.assertIsNotNone(result.extra["queryset"])
        self.assertEqual(result.extra["queryset"].model, Book)

    def test_field_with_choices(self):
        f = User._meta.get_field("status")
        result = FilterSet.filter_for_field(f, "status")
        self.assertIsInstance(result, ChoiceFilter)
        self.assertEqual(result.field_name, "status")
        self.assertTrue("choices" in result.extra)
        self.assertIsNotNone(result.extra["choices"])

    def test_field_that_is_subclassed(self):
        f = User._meta.get_field("first_name")
        result = FilterSet.filter_for_field(f, "first_name")
        self.assertIsInstance(result, CharFilter)

    def test_unknown_field_type_error(self):
        f = NetworkSetting._meta.get_field("mask")

        with self.assertRaises(AssertionError) as excinfo:
            FilterSet.filter_for_field(f, "mask")

        self.assertIn(
            "FilterSet resolved field 'mask' with 'exact' lookup "
            "to an unrecognized field type SubnetMaskField",
            excinfo.exception.args[0],
        )

    def test_symmetrical_selfref_m2m_field(self):
        f = Node._meta.get_field("adjacents")
        result = FilterSet.filter_for_field(f, "adjacents")
        self.assertIsInstance(result, ModelMultipleChoiceFilter)
        self.assertEqual(result.field_name, "adjacents")
        self.assertTrue("queryset" in result.extra)
        self.assertIsNotNone(result.extra["queryset"])
        self.assertEqual(result.extra["queryset"].model, Node)

    def test_non_symmetrical_selfref_m2m_field(self):
        f = DirectedNode._meta.get_field("outbound_nodes")
        result = FilterSet.filter_for_field(f, "outbound_nodes")
        self.assertIsInstance(result, ModelMultipleChoiceFilter)
        self.assertEqual(result.field_name, "outbound_nodes")
        self.assertTrue("queryset" in result.extra)
        self.assertIsNotNone(result.extra["queryset"])
        self.assertEqual(result.extra["queryset"].model, DirectedNode)

    def test_m2m_field_with_through_model(self):
        f = Business._meta.get_field("employees")
        result = FilterSet.filter_for_field(f, "employees")
        self.assertIsInstance(result, ModelMultipleChoiceFilter)
        self.assertEqual(result.field_name, "employees")
        self.assertTrue("queryset" in result.extra)
        self.assertIsNotNone(result.extra["queryset"])
        self.assertEqual(result.extra["queryset"].model, Worker)

    def test_transformed_lookup_expr(self):
        f = Comment._meta.get_field("date")
        result = FilterSet.filter_for_field(f, "date", "year__gte")
        self.assertIsInstance(result, NumberFilter)
        self.assertEqual(result.field_name, "date")

    @override_settings(FILTERS_DEFAULT_LOOKUP_EXPR="icontains")
    def test_modified_default_lookup(self):
        f = User._meta.get_field("username")
        result = FilterSet.filter_for_field(f, "username")
        self.assertIsInstance(result, CharFilter)
        self.assertEqual(result.lookup_expr, "icontains")

    @unittest.skip("todo")
    def test_filter_overrides(self):
        pass


class FilterSetFilterForLookupTests(TestCase):
    def test_filter_for_ISNULL_lookup(self):
        f = Article._meta.get_field("author")
        result, params = FilterSet.filter_for_lookup(f, "isnull")
        self.assertEqual(result, BooleanFilter)
        self.assertDictEqual(params, {})

    def test_filter_for_IN_lookup(self):
        f = Article._meta.get_field("author")
        result, params = FilterSet.filter_for_lookup(f, "in")
        self.assertTrue(issubclass(result, ModelChoiceFilter))
        self.assertTrue(issubclass(result, BaseInFilter))
        self.assertEqual(params["to_field_name"], "id")

    def test_filter_for_RANGE_lookup(self):
        f = Article._meta.get_field("author")
        result, params = FilterSet.filter_for_lookup(f, "range")
        self.assertTrue(issubclass(result, ModelChoiceFilter))
        self.assertTrue(issubclass(result, BaseRangeFilter))
        self.assertEqual(params["to_field_name"], "id")

    def test_isnull_with_filter_overrides(self):
        class OFilterSet(FilterSet):
            class Meta:
                filter_overrides = {
                    models.BooleanField: {
                        "filter_class": BooleanFilter,
                        "extra": lambda f: {
                            "widget": BooleanWidget,
                        },
                    },
                }

        f = Article._meta.get_field("author")
        result, params = OFilterSet.filter_for_lookup(f, "isnull")
        self.assertEqual(result, BooleanFilter)
        self.assertEqual(params["widget"], BooleanWidget)


class ReverseFilterSetFilterForFieldTests(TestCase):
    # Test reverse relationships for `filter_for_field`

    def test_reverse_o2o_relationship(self):
        f = Account._meta.get_field("profile")
        result = FilterSet.filter_for_field(f, "profile")
        self.assertIsInstance(result, ModelChoiceFilter)
        self.assertEqual(result.field_name, "profile")
        self.assertTrue("queryset" in result.extra)
        self.assertIsNotNone(result.extra["queryset"])
        self.assertEqual(result.extra["queryset"].model, Profile)

    def test_reverse_fk_relationship(self):
        f = User._meta.get_field("comments")
        result = FilterSet.filter_for_field(f, "comments")
        self.assertIsInstance(result, ModelMultipleChoiceFilter)
        self.assertEqual(result.field_name, "comments")
        self.assertTrue("queryset" in result.extra)
        self.assertIsNotNone(result.extra["queryset"])
        self.assertEqual(result.extra["queryset"].model, Comment)

    def test_reverse_m2m_relationship(self):
        f = Book._meta.get_field("lovers")
        result = FilterSet.filter_for_field(f, "lovers")
        self.assertIsInstance(result, ModelMultipleChoiceFilter)
        self.assertEqual(result.field_name, "lovers")
        self.assertTrue("queryset" in result.extra)
        self.assertIsNotNone(result.extra["queryset"])
        self.assertEqual(result.extra["queryset"].model, User)

    def test_reverse_non_symmetrical_selfref_m2m_field(self):
        f = DirectedNode._meta.get_field("inbound_nodes")
        result = FilterSet.filter_for_field(f, "inbound_nodes")
        self.assertIsInstance(result, ModelMultipleChoiceFilter)
        self.assertEqual(result.field_name, "inbound_nodes")
        self.assertTrue("queryset" in result.extra)
        self.assertIsNotNone(result.extra["queryset"])
        self.assertEqual(result.extra["queryset"].model, DirectedNode)

    def test_reverse_m2m_field_with_through_model(self):
        f = Worker._meta.get_field("employers")
        result = FilterSet.filter_for_field(f, "employers")
        self.assertIsInstance(result, ModelMultipleChoiceFilter)
        self.assertEqual(result.field_name, "employers")
        self.assertTrue("queryset" in result.extra)
        self.assertIsNotNone(result.extra["queryset"])
        self.assertEqual(result.extra["queryset"].model, Business)

    def test_reverse_relationship_lookup_expr(self):
        f = Book._meta.get_field("lovers")
        result = FilterSet.filter_for_field(f, "lovers", "isnull")
        self.assertIsInstance(result, BooleanFilter)
        self.assertEqual(result.field_name, "lovers")
        self.assertEqual(result.lookup_expr, "isnull")


class FilterSetClassCreationTests(TestCase):
    def test_no_filters(self):
        class F(FilterSet):
            pass

        self.assertEqual(len(F.declared_filters), 0)
        self.assertEqual(len(F.base_filters), 0)

    def test_declaring_filter(self):
        class F(FilterSet):
            username = CharFilter()

        self.assertEqual(len(F.declared_filters), 1)
        self.assertListEqual(list(F.declared_filters), ["username"])
        self.assertEqual(len(F.base_filters), 1)
        self.assertListEqual(list(F.base_filters), ["username"])

    @override_settings(FILTERS_DEFAULT_LOOKUP_EXPR="icontains")
    def test_declaring_filter_other_default_lookup(self):
        class F(FilterSet):
            username = CharFilter()

        self.assertEqual(F.base_filters["username"].lookup_expr, "icontains")

    def test_model_derived(self):
        class F(FilterSet):
            class Meta:
                model = Book
                fields = "__all__"

        self.assertEqual(len(F.declared_filters), 0)
        self.assertEqual(len(F.base_filters), 3)
        self.assertListEqual(list(F.base_filters), ["title", "price", "average_rating"])

    @override_settings(FILTERS_DEFAULT_LOOKUP_EXPR="icontains")
    def test_model_derived_other_default_lookup(self):
        class F(FilterSet):
            class Meta:
                model = Book
                fields = "__all__"

        for filter_ in F.base_filters.values():
            self.assertEqual(filter_.lookup_expr, "icontains")

    def test_model_no_fields_or_exclude(self):
        with self.assertRaises(AssertionError) as excinfo:

            class F(FilterSet):
                class Meta:
                    model = Book

        self.assertIn(
            "Setting 'Meta.model' without either 'Meta.fields' or 'Meta.exclude'",
            str(excinfo.exception),
        )

    def test_model_fields_empty(self):
        class F(FilterSet):
            class Meta:
                model = Book
                fields = []

        self.assertEqual(len(F.declared_filters), 0)
        self.assertEqual(len(F.base_filters), 0)
        self.assertListEqual(list(F.base_filters), [])

    def test_model_exclude_empty(self):
        # equivalent to fields = '__all__'
        class F(FilterSet):
            class Meta:
                model = Book
                exclude = []

        self.assertEqual(len(F.declared_filters), 0)
        self.assertEqual(len(F.base_filters), 3)
        self.assertListEqual(list(F.base_filters), ["title", "price", "average_rating"])

    def test_declared_and_model_derived(self):
        class F(FilterSet):
            username = CharFilter()

            class Meta:
                model = Book
                fields = "__all__"

        self.assertEqual(len(F.declared_filters), 1)
        self.assertEqual(len(F.base_filters), 4)
        self.assertListEqual(
            list(F.base_filters), ["title", "price", "average_rating", "username"]
        )

    def test_meta_fields_with_declared_and_model_derived(self):
        class F(FilterSet):
            username = CharFilter()

            class Meta:
                model = Book
                fields = ("username", "price")

        self.assertEqual(len(F.declared_filters), 1)
        self.assertEqual(len(F.base_filters), 2)
        self.assertListEqual(list(F.base_filters), ["username", "price"])

    def test_meta_fields_dictionary_derived(self):
        class F(FilterSet):
            class Meta:
                model = Book
                fields = {
                    "price": ["exact", "gte", "lte"],
                }

        self.assertEqual(len(F.declared_filters), 0)
        self.assertEqual(len(F.base_filters), 3)

        expected_list = [
            "price",
            "price__gte",
            "price__lte",
        ]
        self.assertCountEqual(list(F.base_filters), expected_list)

    @override_settings(FILTERS_DEFAULT_LOOKUP_EXPR="lte")
    def test_meta_fields_dictionary_derived_other_default_lookup(self):
        class F(FilterSet):
            class Meta:
                model = Book
                fields = {
                    "price": ["exact", "gte", "lte"],
                }

        self.assertEqual(len(F.declared_filters), 0)
        self.assertEqual(len(F.base_filters), 3)

        expected_list = [
            "price__exact",
            "price__gte",
            "price",
        ]
        self.assertCountEqual(list(F.base_filters), expected_list)

    def test_meta_fields_containing_autofield(self):
        class F(FilterSet):
            username = CharFilter()

            class Meta:
                model = Book
                fields = ("id", "username", "price")

        self.assertEqual(len(F.declared_filters), 1)
        self.assertEqual(len(F.base_filters), 3)
        self.assertListEqual(list(F.base_filters), ["id", "username", "price"])

    def test_meta_fields_dictionary_autofield(self):
        class F(FilterSet):
            username = CharFilter()

            class Meta:
                model = User
                fields = {
                    "id": ["exact"],
                    "username": ["exact"],
                }

        self.assertEqual(len(F.declared_filters), 1)
        self.assertEqual(len(F.base_filters), 2)

        expected_list = ["id", "username"]
        self.assertCountEqual(list(F.base_filters), expected_list)

    def test_meta_fields_list_containing_unknown_fields(self):
        msg = "'Meta.fields' must not contain non-model field names: " "other, another"

        with self.assertRaisesMessage(TypeError, msg):

            class F(FilterSet):
                username = CharFilter()

                class Meta:
                    model = Book
                    fields = ("username", "price", "other", "another")

    def test_meta_fields_dict_containing_unknown_fields(self):
        msg = "'Meta.fields' must not contain non-model field names: other"

        with self.assertRaisesMessage(TypeError, msg):

            class F(FilterSet):
                class Meta:
                    model = Book
                    fields = {
                        "id": ["exact"],
                        "title": ["exact"],
                        "other": ["exact"],
                    }

    def test_meta_fields_dict_containing_declarative_alias(self):
        # Meta.fields dict cannot generate lookups for an *aliased* field
        msg = "'Meta.fields' must not contain non-model field names: other"

        with self.assertRaisesMessage(TypeError, msg):

            class F(FilterSet):
                other = CharFilter()

                class Meta:
                    model = Book
                    fields = {
                        "id": ["exact"],
                        "title": ["exact"],
                        "other": ["exact"],
                    }

    def test_meta_fields_invalid_lookup(self):
        # We want to ensure that non existent lookups (or just simple misspellings)
        # throw a useful exception containing the field and lookup expr.
        msg = "Unsupported lookup 'flub' for field 'tests.User.username'."

        with self.assertRaisesMessage(FieldLookupError, msg):

            class F(FilterSet):
                class Meta:
                    model = User
                    fields = {"username": ["flub"]}

    def test_meta_exclude_with_declared_and_declared_wins(self):
        class F(FilterSet):
            username = CharFilter()

            class Meta:
                model = Book
                exclude = ("username", "price")

        self.assertEqual(len(F.declared_filters), 1)
        self.assertEqual(len(F.base_filters), 3)
        self.assertListEqual(
            list(F.base_filters), ["title", "average_rating", "username"]
        )

    def test_meta_fields_and_exclude_and_exclude_wins(self):
        class F(FilterSet):
            username = CharFilter()

            class Meta:
                model = Book
                fields = ("username", "title", "price")
                exclude = ("title",)

        self.assertEqual(len(F.declared_filters), 1)
        self.assertEqual(len(F.base_filters), 2)
        self.assertListEqual(list(F.base_filters), ["username", "price"])

    def test_meta_exclude_with_no_fields(self):
        class F(FilterSet):
            class Meta:
                model = Book
                exclude = ("price",)

        self.assertEqual(len(F.declared_filters), 0)
        self.assertEqual(len(F.base_filters), 2)
        self.assertListEqual(list(F.base_filters), ["title", "average_rating"])

    def test_filterset_class_inheritance(self):
        class F(FilterSet):
            class Meta:
                model = Book
                fields = "__all__"

        class G(F):
            pass

        self.assertEqual(set(F.base_filters), set(G.base_filters))

        class F(FilterSet):
            other = CharFilter

            class Meta:
                model = Book
                fields = "__all__"

        class G(F):
            pass

        self.assertEqual(set(F.base_filters), set(G.base_filters))

    def test_abstract_model_inheritance(self):
        class F(FilterSet):
            class Meta:
                model = Restaurant
                fields = "__all__"

        self.assertEqual(set(F.base_filters), set(["name", "serves_pizza"]))

        class F(FilterSet):
            class Meta:
                model = Restaurant
                fields = ["name", "serves_pizza"]

        self.assertEqual(set(F.base_filters), set(["name", "serves_pizza"]))

    def test_custom_field_gets_filter_from_override(self):
        class F(FilterSet):
            class Meta:
                model = NetworkSetting
                fields = "__all__"

                filter_overrides = {SubnetMaskField: {"filter_class": CharFilter}}

        self.assertEqual(list(F.base_filters.keys()), ["ip", "mask", "cidr"])

    def test_custom_declared_field_no_warning(self):
        class F(FilterSet):
            mask = CharFilter()

            class Meta:
                model = NetworkSetting
                fields = ["mask"]

        self.assertEqual(list(F.base_filters.keys()), ["mask"])

    def test_filterset_for_proxy_model(self):
        class F(FilterSet):
            class Meta:
                model = User
                fields = "__all__"

        class ProxyF(FilterSet):
            class Meta:
                model = AdminUser
                fields = "__all__"

        self.assertEqual(list(F.base_filters), list(ProxyF.base_filters))

    def test_filterset_for_mti_model(self):
        class F(FilterSet):
            class Meta:
                model = Account
                fields = "__all__"

        class FtiF(FilterSet):
            class Meta:
                model = BankAccount
                fields = "__all__"

        # fails due to 'account_ptr' getting picked up
        self.assertEqual(
            list(F.base_filters) + ["amount_saved"], list(FtiF.base_filters)
        )

    def test_declared_filter_disabling(self):
        class Parent(FilterSet):
            f1 = CharFilter()
            f2 = CharFilter()

        class Child(Parent):
            f1 = None

        class Grandchild(Child):
            pass

        self.assertEqual(len(Parent.base_filters), 2)
        self.assertEqual(len(Child.base_filters), 1)
        self.assertEqual(len(Grandchild.base_filters), 1)

    @override_settings(FILTERS_DEFAULT_LOOKUP_EXPR="lt")
    def test_transforms_other_default_lookup(self):
        class F(FilterSet):
            class Meta:
                model = Article
                fields = {
                    "published": ["lt", "year__lt"],
                }

        self.assertEqual(len(F.base_filters), 2)

        expected_list = ["published", "published__year"]
        self.assertCountEqual(list(F.base_filters), expected_list)

    def test_declared_filter_multiple_inheritance(self):
        class A(FilterSet):
            f = CharFilter()

        class B(FilterSet):
            f = NumberFilter()

        class F(A, B):
            pass

        filters = {name: type(f) for name, f in F.declared_filters.items()}
        self.assertEqual(filters, {"f": CharFilter})

    def test_declared_filter_multiple_inheritance_field_ordering(self):
        class Base(FilterSet):
            f1 = CharFilter()
            f2 = CharFilter()

        class A(Base):
            f3 = NumberFilter()

        class B(FilterSet):
            f3 = CharFilter()
            f4 = CharFilter()

        class F(A, B):
            f2 = NumberFilter()
            f5 = CharFilter()

        fields = {name: type(f) for name, f in F.declared_filters.items()}

        # `NumberFilter`s should be the 'winners' in filter name conflicts
        # - `F.f2` should override `Base.F2`
        # - `A.f3` should override `B.f3`
        assert fields == {
            "f1": CharFilter,
            "f2": NumberFilter,
            "f3": NumberFilter,
            "f4": CharFilter,
            "f5": CharFilter,
        }

    def test_filterset_factory(self):
        filterset = filterset_factory(Article)
        self.assertEqual(list(filterset.base_filters), ["name", "published", "author"])

    def test_filterset_factory_fields(self):
        filterset = filterset_factory(Article, fields=["name"])
        self.assertEqual(list(filterset.base_filters), ["name"])

    def test_filterset_factory_base_filter(self):
        class FilterSetBase(FilterSet):
            f1 = CharFilter()
            f2 = CharFilter()

        filterset = filterset_factory(Article, filterset=FilterSetBase)
        self.assertEqual(list(filterset.base_filters), ["name", "published", "author", "f1", "f2"])

    def test_filterset_factory_base_filter_fields(self):
        class FilterSetBase(FilterSet):
            f1 = CharFilter()
            f2 = CharFilter()

        filterset = filterset_factory(Article, filterset=FilterSetBase, fields=["name"])
        self.assertEqual(list(filterset.base_filters), ["name", "f1", "f2"])

    def test_filterset_factory_base_filter_meta_fields(self):
        class FilterSetBase(FilterSet):
            class Meta:
                fields = ["name"]
            f1 = CharFilter()
            f2 = CharFilter()

        filterset = filterset_factory(Article, filterset=FilterSetBase)
        self.assertEqual(list(filterset.base_filters), ["name", "f1", "f2"])

    def test_filterset_factory_base_filter_fields_and_meta_fields(self):
        class FilterSetBase(FilterSet):
            class Meta:
                fields = ["name"]
            f1 = CharFilter()
            f2 = CharFilter()

        filterset = filterset_factory(Article, filterset=FilterSetBase, fields=["author"])
        self.assertEqual(list(filterset.base_filters), ["author", "f1", "f2"])

    def test_filterset_factory_base_filter_meta_inheritance_filter_overrides(self):
        class FilterSetBase(FilterSet):
            class Meta:
                filter_overrides = {
                    models.CharField: {
                        "filter_class": BooleanFilter,
                    },
                }

        filterset = filterset_factory(Article, FilterSetBase)

        f = Article._meta.get_field("author")
        result, params = filterset.filter_for_lookup(f, "isnull")
        self.assertEqual(result, BooleanFilter)

    def test_filterset_factory_base_filter_meta_inheritance_exclude(self):
        class FilterSetBase(FilterSet):
            class Meta:
                exclude = ["published"]

        filterset = filterset_factory(Article, FilterSetBase)
        self.assertEqual(list(filterset.base_filters), ["name", "author"])


class FilterSetInstantiationTests(TestCase):
    class F(FilterSet):
        class Meta:
            model = User
            fields = ["username"]

    def test_creating_instance(self):
        f = self.F()
        self.assertFalse(f.is_bound)
        self.assertIsNotNone(f.queryset)
        self.assertEqual(len(f.filters), len(self.F.base_filters))
        for name, filter_ in f.filters.items():
            self.assertEqual(
                filter_.model, User, "%s does not have model set correctly" % name
            )

    def test_creating_bound_instance(self):
        f = self.F({"username": "username"})
        self.assertTrue(f.is_bound)

    def test_creating_with_queryset(self):
        m = mock.Mock()
        f = self.F(queryset=m)
        self.assertEqual(f.queryset, m)

    def test_creating_with_request(self):
        m = mock.Mock()
        f = self.F(request=m)
        self.assertEqual(f.request, m)

    def test_creating_with_no_data_default(self):
        f = self.F()
        self.assertIsInstance(f.data, MultiValueDict)


class FilterSetQuerysetTests(TestCase):
    class F(FilterSet):
        invalid = CharFilter(method=lambda *args: None)

        class Meta:
            model = User
            fields = ["username", "invalid"]

    def test_filter_queryset_called_once(self):
        m = MockQuerySet()
        f = self.F({"username": "bob"}, queryset=m)

        with mock.patch.object(f, "filter_queryset", wraps=f.filter_queryset) as fn:
            f.qs
            fn.assert_called_once_with(m.all())
            f.qs
            fn.assert_called_once_with(m.all())

    def test_get_form_class_called_once(self):
        f = self.F()

        with mock.patch.object(f, "get_form_class", wraps=f.get_form_class) as fn:
            f.form
            fn.assert_called_once()
            f.form
            fn.assert_called_once()

    def test_qs_caching(self):
        m = mock.Mock()
        f = self.F(queryset=m)

        self.assertIs(f.qs, m.all())
        self.assertIs(f.qs, f.qs)

    def test_form_caching(self):
        f = self.F()

        self.assertIs(f.form, f.form)

    def test_qs_triggers_form_validation(self):
        m = MockQuerySet()
        f = self.F({"username": "bob"}, queryset=m)

        with mock.patch.object(f.form, "full_clean", wraps=f.form.full_clean) as fn:
            fn.assert_not_called()
            f.qs
            fn.assert_called()

    def test_filters_must_return_queryset(self):
        m = MockQuerySet()
        f = self.F({"invalid": "result"}, queryset=m)

        msg = "Expected 'F.invalid' to return a QuerySet, but got a NoneType instead."
        with self.assertRaisesMessage(AssertionError, msg):
            f.qs


# test filter.method here, as it depends on its parent FilterSet
class FilterMethodTests(TestCase):
    def test_none(self):
        # use a mock to bypass bound/unbound method equality
        class TestFilter(Filter):
            filter = mock.Mock()

        f = TestFilter(method=None)
        self.assertIsNone(f.method)

        # passing method=None should not modify filter function
        self.assertIs(f.filter, TestFilter.filter)

    def test_method_name(self):
        class F(FilterSet):
            f = Filter(method="filter_f")

            def filter_f(self, qs, name, value):
                pass

        f = F({}, queryset=User.objects.all())
        self.assertEqual(f.filters["f"].method, "filter_f")
        self.assertEqual(f.filters["f"].filter.method, f.filter_f)
        self.assertIsInstance(f.filters["f"].filter, FilterMethod)

    def test_method_callable(self):
        def filter_f(qs, name, value):
            pass

        class F(FilterSet):
            f = Filter(method=filter_f)

        f = F({}, queryset=User.objects.all())
        self.assertEqual(f.filters["f"].method, filter_f)
        self.assertEqual(f.filters["f"].filter.method, filter_f)
        self.assertIsInstance(f.filters["f"].filter, FilterMethod)

    def test_request_available_during_method_called(self):
        class F(FilterSet):
            f = Filter(method="filter_f")

            def filter_f(self, qs, name, value):
                # call mock request object to prove self.request can be accessed
                self.request()

        m = mock.Mock()
        f = F({}, queryset=User.objects.all(), request=m)
        # call the filter
        f.filters["f"].filter.method(User.objects.all(), "f", "")
        m.assert_called_once_with()

    def test_method_with_overridden_filter(self):
        # Some filter classes override the base filter() method. We need
        # to ensure that passing a method argument still works correctly
        class F(FilterSet):
            f = DateRangeFilter(method="filter_f")

            def filter_f(self, qs, name, value):
                pass

        f = F({}, queryset=User.objects.all())
        self.assertEqual(f.filters["f"].method, "filter_f")
        self.assertEqual(f.filters["f"].filter.method, f.filter_f)

    def test_parent_unresolvable(self):
        f = Filter(method="filter_f")
        with self.assertRaises(AssertionError) as w:
            f.filter(User.objects.all(), 0)

        self.assertIn("'None'", str(w.exception))
        self.assertIn("parent", str(w.exception))
        self.assertIn("filter_f", str(w.exception))

    def test_method_self_is_parent(self):
        # Ensure the method isn't 're-parented' on the `FilterMethod` helper class.
        # Filter methods should have access to the filterset's properties.
        request = MockQuerySet()

        class F(FilterSet):
            f = CharFilter(method="filter_f")

            class Meta:
                model = User
                fields = []

            def filter_f(inner_self, qs, name, value):
                self.assertIsInstance(inner_self, F)
                self.assertIs(inner_self.request, request)
                return qs

        F({"f": "foo"}, request=request, queryset=User.objects.all()).qs

    def test_method_unresolvable(self):
        class F(FilterSet):
            f = Filter(method="filter_f")

        f = F({}, queryset=User.objects.all())

        with self.assertRaises(AssertionError) as w:
            f.filters["f"].filter(User.objects.all(), 0)

        self.assertIn("%s.%s" % (F.__module__, F.__name__), str(w.exception))
        self.assertIn(".filter_f()", str(w.exception))

    def test_method_uncallable(self):
        class F(FilterSet):
            f = Filter(method="filter_f")
            filter_f = 4

        f = F({}, queryset=User.objects.all())

        with self.assertRaises(AssertionError) as w:
            f.filters["f"].filter(User.objects.all(), 0)

        self.assertIn("%s.%s" % (F.__module__, F.__name__), str(w.exception))
        self.assertIn(".filter_f()", str(w.exception))

    def test_method_set_unset(self):
        # use a mock to bypass bound/unbound method equality
        class TestFilter(Filter):
            filter = mock.Mock()

        f = TestFilter(method="filter_f")
        self.assertEqual(f.method, "filter_f")
        self.assertIsInstance(f.filter, FilterMethod)

        # setting None should revert to Filter.filter
        f.method = None
        self.assertIsNone(f.method)
        self.assertIs(f.filter, TestFilter.filter)


class MiscFilterSetTests(TestCase):
    def test_no__getitem__(self):
        # The DTL processes variable lookups by the following rules:
        # https://docs.djangoproject.com/en/stable/ref/templates/language/#variables
        # A __getitem__ implementation precedes normal attribute access, and in
        # the case of #58, will force the queryset to evaluate when it should
        # not (eg, when rendering a blank form).
        self.assertFalse(hasattr(FilterSet, "__getitem__"))

    def test_no_qs_proxying(self):
        # The FilterSet should not proxy .qs methods - just access .qs directly
        self.assertFalse(hasattr(FilterSet, "__len__"))
        self.assertFalse(hasattr(FilterSet, "__iter__"))



import inspect

from django.test import TestCase

from django_filters.rest_framework import filters
from django_filters.widgets import BooleanWidget


class ModuleImportTests(TestCase):
    def is_filter(self, name, value):
        return isinstance(value, type) and issubclass(value, filters.Filter)

    def test_imports(self):
        # msg = "Expected `filters.%s` to be imported in `filters.__all__`"
        filter_classes = [
            key
            for key, value in inspect.getmembers(filters)
            if isinstance(value, type) and issubclass(value, filters.Filter)
        ]

        # sanity check
        self.assertIn("Filter", filter_classes)
        self.assertIn("BooleanFilter", filter_classes)

        for f in filter_classes:
            self.assertIn(f, filters.__all__)


class BooleanFilterTests(TestCase):
    def test_widget(self):
        # Ensure that `BooleanFilter` uses the correct widget when importing
        # from `rest_framework.filters`.
        f = filters.BooleanFilter()

        self.assertEqual(f.extra["widget"], BooleanWidget)



from django.db import models
from django.utils.translation import gettext_lazy as _


class BasicModel(models.Model):
    text = models.CharField(
        max_length=100,
        verbose_name=_("Text comes here"),
        help_text=_("Text description."),
    )


class BaseFilterableItem(models.Model):
    text = models.CharField(max_length=100)


class FilterableItem(BaseFilterableItem):
    decimal = models.DecimalField(max_digits=4, decimal_places=2)
    date = models.DateField()


class DjangoFilterOrderingModel(models.Model):
    date = models.DateField()
    text = models.CharField(max_length=10)

    class Meta:
        ordering = ["-date"]


class CategoryItem(BaseFilterableItem):
    category = models.CharField(
        max_length=10, choices=(("home", "Home"), ("office", "Office"))
    )



# ensure package/conf is importable
from django_filters.conf import DEFAULTS

DATABASES = {
    "default": {
        "ENGINE": "django.db.backends.sqlite3",
        "NAME": ":memory:",
    },
}

INSTALLED_APPS = (
    "django.contrib.contenttypes",
    "django.contrib.staticfiles",
    "django.contrib.auth",
    "rest_framework",
    "django_filters",
    "tests.rest_framework",
    "tests",
)

MIDDLEWARE = []

ROOT_URLCONF = "tests.urls"

USE_TZ = True

TIME_ZONE = "UTC"

SECRET_KEY = "foobar"

TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "APP_DIRS": True,
    }
]


STATIC_URL = "/static/"


# XMLTestRunner output
TEST_OUTPUT_DIR = ".xmlcoverage"


# help verify that DEFAULTS is importable from conf.
def FILTERS_VERBOSE_LOOKUPS():
    return DEFAULTS["VERBOSE_LOOKUPS"]


DEFAULT_AUTO_FIELD = "django.db.models.BigAutoField"



from django import forms
from django.test import TestCase, override_settings

from django_filters.filters import CharFilter, ChoiceFilter
from django_filters.filterset import FilterSet

from .models import MANAGER, REGULAR, STATUS_CHOICES, Book, ManagerGroup, User


class FilterSetFormTests(TestCase):
    def test_form_from_empty_filterset(self):
        class F(FilterSet):
            pass

        f = F(queryset=Book.objects.all()).form
        self.assertIsInstance(f, forms.Form)

    def test_form(self):
        class F(FilterSet):
            class Meta:
                model = Book
                fields = ("title",)

        f = F().form
        self.assertIsInstance(f, forms.Form)
        self.assertEqual(list(f.fields), ["title"])

    def test_custom_form(self):
        class MyForm(forms.Form):
            pass

        class F(FilterSet):
            class Meta:
                model = Book
                fields = "__all__"
                form = MyForm

        f = F().form
        self.assertIsInstance(f, MyForm)

    def test_form_prefix(self):
        class F(FilterSet):
            class Meta:
                model = Book
                fields = ("title",)

        f = F().form
        self.assertIsNone(f.prefix)

        f = F(prefix="prefix").form
        self.assertEqual(f.prefix, "prefix")

    def test_form_fields(self):
        class F(FilterSet):
            class Meta:
                model = User
                fields = ["status"]

        f = F().form
        self.assertEqual(len(f.fields), 1)
        self.assertIn("status", f.fields)
        self.assertSequenceEqual(
            list(f.fields["status"].choices), (("", "---------"),) + STATUS_CHOICES
        )

    def test_form_fields_exclusion(self):
        class F(FilterSet):
            title = CharFilter(exclude=True)

            class Meta:
                model = Book
                fields = ("title",)

        f = F().form
        self.assertEqual(f.fields["title"].label, "Exclude title")

    def test_complex_form_fields(self):
        class F(FilterSet):
            username = CharFilter(label="Filter for users with username")
            exclude_username = CharFilter(
                field_name="username", lookup_expr="iexact", exclude=True
            )

            class Meta:
                model = User
                fields = {
                    "status": ["exact", "lt", "gt"],
                    "favorite_books__title": ["iexact", "in"],
                    "manager_of__users__username": ["exact"],
                }

        fields = F().form.fields
        self.assertEqual(fields["username"].label, "Filter for users with username")
        self.assertEqual(fields["exclude_username"].label, "Exclude username")
        self.assertEqual(fields["status"].label, "Status")
        self.assertEqual(fields["status__lt"].label, "Status is less than")
        self.assertEqual(fields["status__gt"].label, "Status is greater than")
        self.assertEqual(
            fields["favorite_books__title__iexact"].label, "Favorite books title"
        )
        self.assertEqual(
            fields["favorite_books__title__in"].label, "Favorite books title is in"
        )
        self.assertEqual(
            fields["manager_of__users__username"].label, "Manager of users username"
        )

    def test_form_fields_using_widget(self):
        class F(FilterSet):
            status = ChoiceFilter(
                widget=forms.RadioSelect, choices=STATUS_CHOICES, empty_label=None
            )

            class Meta:
                model = User
                fields = ["status", "username"]

        f = F().form
        self.assertEqual(len(f.fields), 2)
        self.assertIn("status", f.fields)
        self.assertIn("username", f.fields)
        self.assertSequenceEqual(list(f.fields["status"].choices), STATUS_CHOICES)
        self.assertIsInstance(f.fields["status"].widget, forms.RadioSelect)

    def test_form_field_with_custom_label(self):
        class F(FilterSet):
            title = CharFilter(label="Book title")

            class Meta:
                model = Book
                fields = ("title",)

        f = F().form
        self.assertEqual(f.fields["title"].label, "Book title")
        self.assertEqual(f["title"].label, "Book title")

    def test_form_field_with_manual_name(self):
        class F(FilterSet):
            book_title = CharFilter(field_name="title")

            class Meta:
                model = Book
                fields = ("book_title",)

        f = F().form
        self.assertEqual(f.fields["book_title"].label, "Title")
        self.assertEqual(f["book_title"].label, "Title")

    def test_form_field_with_manual_name_and_label(self):
        class F(FilterSet):
            f1 = CharFilter(field_name="title", label="Book title")

            class Meta:
                model = Book
                fields = ("f1",)

        f = F().form
        self.assertEqual(f.fields["f1"].label, "Book title")
        self.assertEqual(f["f1"].label, "Book title")

    def test_filter_with_initial(self):
        class F(FilterSet):
            status = ChoiceFilter(choices=STATUS_CHOICES, initial=1)

            class Meta:
                model = User
                fields = ["status"]

        f = F().form
        self.assertEqual(f.fields["status"].initial, 1)

    def test_form_is_not_bound(self):
        class F(FilterSet):
            class Meta:
                model = Book
                fields = ("title",)

        f = F().form
        self.assertFalse(f.is_bound)
        self.assertEqual(f.data, {})

    def test_form_is_bound(self):
        class F(FilterSet):
            class Meta:
                model = Book
                fields = ("title",)

        f = F({"title": "Some book"}).form
        self.assertTrue(f.is_bound)
        self.assertEqual(f.data, {"title": "Some book"})

    def test_limit_choices_to(self):
        User.objects.create(username="inactive", is_active=False, status=REGULAR)
        User.objects.create(username="active", is_active=True, status=REGULAR)
        User.objects.create(username="manager", is_active=False, status=MANAGER)

        class F(FilterSet):
            class Meta:
                model = ManagerGroup
                fields = ["users", "manager"]

        f = F().form
        self.assertEqual(list(f.fields["users"].choices), [(2, "active")])
        self.assertEqual(
            list(f.fields["manager"].choices), [("", "---------"), (3, "manager")]
        )

    def test_disabled_help_text(self):
        class F(FilterSet):
            class Meta:
                model = Book
                fields = {
                    # 'in' lookups are CSV-based, which have a `help_text`.
                    "title": ["in"]
                }

        self.assertEqual(
            F().form.fields["title__in"].help_text,
            "Multiple values may be separated by commas.",
        )

        with override_settings(FILTERS_DISABLE_HELP_TEXT=True):

            self.assertEqual(F().form.fields["title__in"].help_text, "")


class FilterSetValidityTests(TestCase):
    class F(FilterSet):
        class Meta:
            model = Book
            fields = ["title", "price"]

    def test_not_bound(self):
        f = self.F()

        self.assertFalse(f.is_bound)
        self.assertFalse(f.is_valid())
        self.assertEqual(f.data, {})
        self.assertEqual(f.errors, {})

    def test_is_bound_and_valid(self):
        f = self.F({"title": "Some book"})

        self.assertTrue(f.is_bound)
        self.assertTrue(f.is_valid())
        self.assertEqual(f.data, {"title": "Some book"})
        self.assertEqual(f.errors, {})

    def test_is_bound_and_not_valid(self):
        f = self.F({"price": "four dollars"})

        self.assertTrue(f.is_bound)
        self.assertFalse(f.is_valid())
        self.assertEqual(f.data, {"price": "four dollars"})
        self.assertEqual(f.errors, {"price": ["Enter a number."]})

    def test_number_filter_max_value_validation(self):
        class F(FilterSet):
            class Meta:
                model = Book
                fields = ["average_rating"]

        f = F({"average_rating": "1E1001"})
        self.assertTrue(f.is_bound)
        self.assertFalse(f.is_valid())
        self.assertEqual(
            f.errors,
            {"average_rating": ["Ensure this value is less than or equal to 1e+50."]},
        )



import contextlib
import datetime
import unittest
from operator import attrgetter
from unittest import mock

import django
from django import forms
from django.http import QueryDict
from django.test import TestCase, override_settings
from django.utils import timezone
from django.utils.timezone import make_aware, now

from django_filters.filters import (
    AllValuesFilter,
    AllValuesMultipleFilter,
    BaseInFilter,
    CharFilter,
    ChoiceFilter,
    DateFromToRangeFilter,
    DateRangeFilter,
    DateTimeFromToRangeFilter,
    DurationFilter,
    IsoDateTimeFromToRangeFilter,
    LookupChoiceFilter,
    ModelChoiceFilter,
    ModelMultipleChoiceFilter,
    MultipleChoiceFilter,
    OrderingFilter,
    RangeFilter,
    TimeRangeFilter,
    TypedMultipleChoiceFilter,
)
from django_filters.filterset import FilterSet
from django_filters.widgets import QueryArrayWidget

from .models import (
    STATUS_CHOICES,
    Account,
    Article,
    BankAccount,
    Book,
    Comment,
    Company,
    DirectedNode,
    Location,
    Node,
    Profile,
    SpacewalkRecord,
    User,
)
from .utils import MockQuerySet


class CharFilterTests(TestCase):
    def test_filtering(self):
        b1 = Book.objects.create(title="Ender's Game", price="1.00", average_rating=3.0)
        b2 = Book.objects.create(title="Rainbow Six", price="1.00", average_rating=3.0)
        b3 = Book.objects.create(title="Snowcrash", price="1.00", average_rating=3.0)

        class F(FilterSet):
            class Meta:
                model = Book
                fields = ["title"]

        qs = Book.objects.all()
        f = F(queryset=qs)
        self.assertQuerySetEqual(
            f.qs, [b1.pk, b2.pk, b3.pk], lambda o: o.pk, ordered=False
        )
        f = F({"title": "Snowcrash"}, queryset=qs)
        self.assertQuerySetEqual(f.qs, [b3.pk], lambda o: o.pk)


class IntegerFilterTest(TestCase):
    def test_filtering(self):
        default_values = {
            "in_good_standing": True,
            "friendly": False,
        }
        b1 = BankAccount.objects.create(amount_saved=0, **default_values)
        b2 = BankAccount.objects.create(amount_saved=3, **default_values)
        b3 = BankAccount.objects.create(amount_saved=10, **default_values)

        class F(FilterSet):
            class Meta:
                model = BankAccount
                fields = ["amount_saved"]

        qs = BankAccount.objects.all()
        f = F(queryset=qs)
        self.assertQuerySetEqual(
            f.qs, [b1.pk, b2.pk, b3.pk], lambda o: o.pk, ordered=False
        )
        f = F({"amount_saved": "10"}, queryset=qs)
        self.assertQuerySetEqual(f.qs, [b3.pk], lambda o: o.pk)
        f = F({"amount_saved": "0"}, queryset=qs)
        self.assertQuerySetEqual(f.qs, [b1.pk], lambda o: o.pk)


class BooleanFilterTests(TestCase):
    def test_filtering(self):
        User.objects.create(username="alex", is_active=False)
        User.objects.create(username="jacob", is_active=True)
        User.objects.create(username="aaron", is_active=False)

        class F(FilterSet):
            class Meta:
                model = User
                fields = ["is_active"]

        qs = User.objects.all()

        # '2' and '3' are how the field expects the data from the browser
        f = F({"is_active": "2"}, queryset=qs)
        self.assertQuerySetEqual(f.qs, ["jacob"], lambda o: o.username, False)

        f = F({"is_active": "3"}, queryset=qs)
        self.assertQuerySetEqual(f.qs, ["alex", "aaron"], lambda o: o.username, False)

        f = F({"is_active": "1"}, queryset=qs)
        self.assertQuerySetEqual(
            f.qs, ["alex", "aaron", "jacob"], lambda o: o.username, False
        )


class ChoiceFilterTests(TestCase):
    @classmethod
    def setUpTestData(cls):
        User.objects.create(username="alex", status=1)
        User.objects.create(username="jacob", status=2)
        User.objects.create(username="aaron", status=2)
        User.objects.create(username="carl", status=0)

        Article.objects.create(author_id=1, published=now())
        Article.objects.create(author_id=2, published=now())
        Article.objects.create(author_id=3, published=now())
        Article.objects.create(author_id=4, published=now())
        Article.objects.create(author_id=None, published=now())

    def test_filtering(self):
        class F(FilterSet):
            class Meta:
                model = User
                fields = ["status"]

        f = F()
        self.assertQuerySetEqual(
            f.qs, ["aaron", "alex", "jacob", "carl"], lambda o: o.username, False
        )
        f = F({"status": "1"})
        self.assertQuerySetEqual(f.qs, ["alex"], lambda o: o.username, False)

        f = F({"status": "2"})
        self.assertQuerySetEqual(f.qs, ["jacob", "aaron"], lambda o: o.username, False)

        f = F({"status": "0"})
        self.assertQuerySetEqual(f.qs, ["carl"], lambda o: o.username, False)

    def test_filtering_on_explicitly_defined_field(self):
        """
        Test for #30.

        If you explicitly declare ChoiceFilter fields you **MUST** pass `choices`.
        """

        class F(FilterSet):
            status = ChoiceFilter(choices=STATUS_CHOICES)

            class Meta:
                model = User
                fields = ["status"]

        f = F()
        self.assertQuerySetEqual(
            f.qs, ["aaron", "alex", "jacob", "carl"], lambda o: o.username, False
        )
        f = F({"status": "1"})
        self.assertQuerySetEqual(f.qs, ["alex"], lambda o: o.username, False)

        f = F({"status": "2"})
        self.assertQuerySetEqual(f.qs, ["jacob", "aaron"], lambda o: o.username, False)

        f = F({"status": "0"})
        self.assertQuerySetEqual(f.qs, ["carl"], lambda o: o.username, False)

    def test_filtering_on_empty_choice(self):
        class F(FilterSet):
            class Meta:
                model = User
                fields = ["status"]

        f = F({"status": ""})
        self.assertQuerySetEqual(
            f.qs, ["aaron", "alex", "jacob", "carl"], lambda o: o.username, False
        )

    def test_filtering_on_null_choice(self):
        choices = [(u.pk, str(u)) for u in User.objects.order_by("id")]

        class F(FilterSet):
            author = ChoiceFilter(
                choices=choices,
                null_value="null",
                null_label="NULL",
            )

            class Meta:
                model = Article
                fields = ["author"]

        # sanity check to make sure the filter is setup correctly
        f = F({"author": "1"})
        self.assertQuerySetEqual(f.qs, ["alex"], lambda o: str(o.author), False)

        f = F({"author": "null"})
        self.assertQuerySetEqual(f.qs, [None], lambda o: o.author, False)


class MultipleChoiceFilterTests(TestCase):
    def test_filtering(self):
        User.objects.create(username="alex", status=1)
        User.objects.create(username="jacob", status=2)
        User.objects.create(username="aaron", status=2)
        User.objects.create(username="carl", status=0)

        class F(FilterSet):
            status = MultipleChoiceFilter(choices=STATUS_CHOICES)

            class Meta:
                model = User
                fields = ["status"]

        qs = User.objects.all().order_by("username")
        f = F(queryset=qs)
        self.assertQuerySetEqual(
            f.qs, ["aaron", "jacob", "alex", "carl"], lambda o: o.username, False
        )

        f = F({"status": ["0"]}, queryset=qs)
        self.assertQuerySetEqual(f.qs, ["carl"], lambda o: o.username)

        f = F({"status": ["0", "1"]}, queryset=qs)
        self.assertQuerySetEqual(f.qs, ["alex", "carl"], lambda o: o.username)

        f = F({"status": ["0", "1", "2"]}, queryset=qs)
        self.assertQuerySetEqual(
            f.qs, ["aaron", "alex", "carl", "jacob"], lambda o: o.username
        )

    def test_filtering_on_null_choice(self):
        User.objects.create(username="alex", status=1)
        User.objects.create(username="jacob", status=2)
        User.objects.create(username="aaron", status=2)
        User.objects.create(username="carl", status=0)

        Article.objects.create(author_id=1, published=now())
        Article.objects.create(author_id=2, published=now())
        Article.objects.create(author_id=3, published=now())
        Article.objects.create(author_id=4, published=now())
        Article.objects.create(author_id=None, published=now())

        choices = [(u.pk, str(u)) for u in User.objects.order_by("id")]

        class F(FilterSet):
            author = MultipleChoiceFilter(
                choices=choices,
                null_value="null",
                null_label="NULL",
            )

            class Meta:
                model = Article
                fields = ["author"]

        # sanity check to make sure the filter is setup correctly
        f = F({"author": ["1"]})
        self.assertQuerySetEqual(f.qs, ["alex"], lambda o: str(o.author), False)

        f = F({"author": ["null"]})
        self.assertQuerySetEqual(f.qs, [None], lambda o: o.author, False)

        f = F({"author": ["1", "null"]})
        self.assertQuerySetEqual(
            f.qs, ["alex", None], lambda o: o.author and str(o.author), False
        )


class TypedMultipleChoiceFilterTests(TestCase):
    def test_filtering(self):
        User.objects.create(username="alex", status=1)
        User.objects.create(username="jacob", status=2)
        User.objects.create(username="aaron", status=2)
        User.objects.create(username="carl", status=0)

        class F(FilterSet):
            status = TypedMultipleChoiceFilter(
                choices=STATUS_CHOICES, coerce=lambda x: x[0:2]
            )

            class Meta:
                model = User
                fields = ["status"]

        qs = User.objects.all().order_by("username")
        f = F(queryset=qs)
        self.assertQuerySetEqual(
            f.qs, ["aa", "ja", "al", "ca"], lambda o: o.username[0:2], False
        )

        f = F({"status": ["0"]}, queryset=qs)
        self.assertQuerySetEqual(f.qs, ["ca"], lambda o: o.username[0:2])

        f = F({"status": ["0", "1"]}, queryset=qs)
        self.assertQuerySetEqual(f.qs, ["al", "ca"], lambda o: o.username[0:2])

        f = F({"status": ["0", "1", "2"]}, queryset=qs)
        self.assertQuerySetEqual(
            f.qs, ["aa", "al", "ca", "ja"], lambda o: o.username[0:2]
        )


class DateFilterTests(TestCase):
    def test_filtering(self):
        today = now().date()
        timestamp = now().time().replace(microsecond=0)
        last_week = today - datetime.timedelta(days=7)
        check_date = str(last_week)
        u = User.objects.create(username="alex")
        Comment.objects.create(author=u, time=timestamp, date=today)
        Comment.objects.create(author=u, time=timestamp, date=last_week)
        Comment.objects.create(author=u, time=timestamp, date=today)
        Comment.objects.create(author=u, time=timestamp, date=last_week)

        class F(FilterSet):
            class Meta:
                model = Comment
                fields = ["date"]

        f = F({"date": check_date}, queryset=Comment.objects.all())
        self.assertEqual(len(f.qs), 2)
        self.assertQuerySetEqual(f.qs, [2, 4], lambda o: o.pk, False)


class TimeFilterTests(TestCase):
    def test_filtering(self):
        today = now().date()
        now_time = now().time().replace(microsecond=0)
        ten_min_ago = now() - datetime.timedelta(minutes=10)
        fixed_time = ten_min_ago.time().replace(microsecond=0)
        check_time = str(fixed_time)
        u = User.objects.create(username="alex")
        Comment.objects.create(author=u, time=now_time, date=today)
        Comment.objects.create(author=u, time=fixed_time, date=today)
        Comment.objects.create(author=u, time=now_time, date=today)
        Comment.objects.create(author=u, time=fixed_time, date=today)

        class F(FilterSet):
            class Meta:
                model = Comment
                fields = ["time"]

        f = F({"time": check_time}, queryset=Comment.objects.all())
        self.assertEqual(len(f.qs), 2)
        self.assertQuerySetEqual(f.qs, [2, 4], lambda o: o.pk, False)


class DateTimeFilterTests(TestCase):
    def test_filtering(self):
        now_dt = now()
        ten_min_ago = now_dt - datetime.timedelta(minutes=10)
        one_day_ago = now_dt - datetime.timedelta(days=1)
        u = User.objects.create(username="alex")
        Article.objects.create(author=u, published=now_dt)
        Article.objects.create(author=u, published=ten_min_ago)
        Article.objects.create(author=u, published=one_day_ago)

        tz = timezone.get_current_timezone()
        # make naive, like a browser would send
        local_ten_min_ago = timezone.make_naive(ten_min_ago, tz)
        check_dt = str(local_ten_min_ago)

        class F(FilterSet):
            class Meta:
                model = Article
                fields = ["published"]

        qs = Article.objects.all()
        f = F({"published": ten_min_ago}, queryset=qs)
        self.assertEqual(len(f.qs), 1)
        self.assertQuerySetEqual(f.qs, [2], lambda o: o.pk)

        # this is how it would come through a browser
        f = F({"published": check_dt}, queryset=qs)
        self.assertEqual(
            len(f.qs), 1, "%s isn't matching %s when cleaned" % (check_dt, ten_min_ago)
        )
        self.assertQuerySetEqual(f.qs, [2], lambda o: o.pk)


class DurationFilterTests(TestCase):
    """Duration filter tests.

    The preferred format for durations in Django is '%d %H:%M:%S.%f'.
    See django.utils.dateparse.parse_duration

    Django is not fully ISO 8601 compliant (yet): year, month, and
    week designators are not supported, so a duration string
    like "P3Y6M4DT12H30M5S" cannot be used.
    See https://en.wikipedia.org/wiki/ISO_8601#Durations

    """

    def setUp(self):
        self.r1 = SpacewalkRecord.objects.create(
            astronaut="Anatoly Solovyev",
            duration=datetime.timedelta(hours=82, minutes=22),
        )
        self.r2 = SpacewalkRecord.objects.create(
            astronaut="Michael Lopez-Alegria",
            duration=datetime.timedelta(hours=67, minutes=40),
        )
        self.r3 = SpacewalkRecord.objects.create(
            astronaut="Jerry L. Ross", duration=datetime.timedelta(hours=58, minutes=32)
        )
        self.r4 = SpacewalkRecord.objects.create(
            astronaut="John M. Grunsfeld",
            duration=datetime.timedelta(hours=58, minutes=30),
        )
        self.r5 = SpacewalkRecord.objects.create(
            astronaut="Richard Mastracchio",
            duration=datetime.timedelta(hours=53, minutes=4),
        )

    def test_filtering(self):
        class F(FilterSet):
            class Meta:
                model = SpacewalkRecord
                fields = ["duration"]

        qs = SpacewalkRecord.objects.all()

        # Django style: 3 days, 10 hours, 22 minutes.
        f = F({"duration": "3 10:22:00"}, queryset=qs)
        self.assertQuerySetEqual(f.qs, [self.r1], lambda x: x)

        # ISO 8601: 3 days, 10 hours, 22 minutes.
        f = F({"duration": "P3DT10H22M"}, queryset=qs)
        self.assertQuerySetEqual(f.qs, [self.r1], lambda x: x)

        # Django style: 82 hours, 22 minutes.
        f = F({"duration": "82:22:00"}, queryset=qs)
        self.assertQuerySetEqual(f.qs, [self.r1], lambda x: x)

        # ISO 8601: 82 hours, 22 minutes.
        f = F({"duration": "PT82H22M"}, queryset=qs)
        self.assertQuerySetEqual(f.qs, [self.r1], lambda x: x)

    def test_filtering_with_single_lookup_expr_dictionary(self):
        class F(FilterSet):
            class Meta:
                model = SpacewalkRecord
                fields = {"duration": ["gt", "gte", "lt", "lte"]}

        qs = SpacewalkRecord.objects.order_by("-duration")

        f = F({"duration__gt": "PT58H30M"}, queryset=qs)
        self.assertQuerySetEqual(f.qs, [self.r1, self.r2, self.r3], lambda x: x)

        f = F({"duration__gte": "PT58H30M"}, queryset=qs)
        self.assertQuerySetEqual(
            f.qs, [self.r1, self.r2, self.r3, self.r4], lambda x: x
        )

        f = F({"duration__lt": "PT58H30M"}, queryset=qs)
        self.assertQuerySetEqual(f.qs, [self.r5], lambda x: x)

        f = F({"duration__lte": "PT58H30M"}, queryset=qs)
        self.assertQuerySetEqual(f.qs, [self.r4, self.r5], lambda x: x)

    def test_filtering_with_multiple_lookup_exprs(self):
        class F(FilterSet):
            min_duration = DurationFilter(field_name="duration", lookup_expr="gte")
            max_duration = DurationFilter(field_name="duration", lookup_expr="lte")

            class Meta:
                model = SpacewalkRecord
                fields = "__all__"

        qs = SpacewalkRecord.objects.order_by("duration")

        f = F({"min_duration": "PT55H", "max_duration": "PT60H"}, queryset=qs)
        self.assertQuerySetEqual(f.qs, [self.r4, self.r3], lambda x: x)


class ModelChoiceFilterTests(TestCase):
    def test_filtering(self):
        alex = User.objects.create(username="alex")
        jacob = User.objects.create(username="jacob")
        date = now().date()
        time = now().time()
        Comment.objects.create(author=jacob, time=time, date=date)
        Comment.objects.create(author=alex, time=time, date=date)
        Comment.objects.create(author=jacob, time=time, date=date)

        class F(FilterSet):
            class Meta:
                model = Comment
                fields = ["author"]

        qs = Comment.objects.all()
        f = F({"author": jacob.pk}, queryset=qs)
        self.assertQuerySetEqual(f.qs, [1, 3], lambda o: o.pk, False)

    @override_settings(FILTERS_NULL_CHOICE_LABEL="No Author")
    def test_filtering_null(self):
        Article.objects.create(published=now())
        alex = User.objects.create(username="alex")
        Article.objects.create(author=alex, published=now())

        class F(FilterSet):
            class Meta:
                model = Article
                fields = ["author", "name"]

        qs = Article.objects.all()
        f = F({"author": "null"}, queryset=qs)
        self.assertQuerySetEqual(f.qs, [None], lambda o: o.author, False)

    def test_callable_queryset(self):
        # Sanity check for callable queryset arguments.
        # Ensure that nothing is improperly cached
        User.objects.create(username="alex")
        jacob = User.objects.create(username="jacob")
        aaron = User.objects.create(username="aaron")

        def users(request):
            return User.objects.filter(pk__lt=request.user.pk)

        class F(FilterSet):
            author = ModelChoiceFilter(field_name="author", queryset=users)

            class Meta:
                model = Comment
                fields = ["author"]

        qs = Comment.objects.all()
        request = mock.Mock()

        request.user = jacob
        f = F(queryset=qs, request=request).filters["author"].field
        self.assertQuerySetEqual(f.queryset, [1], lambda o: o.pk, False)

        request.user = aaron
        f = F(queryset=qs, request=request).filters["author"].field
        self.assertQuerySetEqual(f.queryset, [1, 2], lambda o: o.pk, False)


class ModelMultipleChoiceFilterTests(TestCase):
    def setUp(self):
        alex = User.objects.create(username="alex")
        User.objects.create(username="jacob")
        aaron = User.objects.create(username="aaron")
        b1 = Book.objects.create(title="Ender's Game", price="1.00", average_rating=3.0)
        b2 = Book.objects.create(title="Rainbow Six", price="1.00", average_rating=3.0)
        b3 = Book.objects.create(title="Snowcrash", price="1.00", average_rating=3.0)
        Book.objects.create(
            title="Stranger in a Strage Land", price="1.00", average_rating=3.0
        )
        alex.favorite_books.add(b1, b2)
        aaron.favorite_books.add(b1, b3)

        self.alex = alex

    def test_filtering(self):
        class F(FilterSet):
            class Meta:
                model = User
                fields = ["favorite_books"]

        qs = User.objects.all().order_by("username")
        f = F({"favorite_books": ["1"]}, queryset=qs)
        self.assertQuerySetEqual(f.qs, ["aaron", "alex"], lambda o: o.username)

        f = F({"favorite_books": ["1", "3"]}, queryset=qs)
        self.assertQuerySetEqual(f.qs, ["aaron", "alex"], lambda o: o.username)

        f = F({"favorite_books": ["2"]}, queryset=qs)
        self.assertQuerySetEqual(f.qs, ["alex"], lambda o: o.username)

        f = F({"favorite_books": ["4"]}, queryset=qs)
        self.assertQuerySetEqual(f.qs, [], lambda o: o.username)

    @override_settings(FILTERS_NULL_CHOICE_LABEL="No Favorites")
    def test_filtering_null(self):
        class F(FilterSet):
            class Meta:
                model = User
                fields = ["favorite_books"]

        qs = User.objects.all()
        f = F({"favorite_books": ["null"]}, queryset=qs)

        self.assertQuerySetEqual(f.qs, ["jacob"], lambda o: o.username)

    def test_filtering_dictionary(self):
        class F(FilterSet):
            class Meta:
                model = User
                fields = {"favorite_books": ["exact"]}

        qs = User.objects.all().order_by("username")
        f = F({"favorite_books": ["1"]}, queryset=qs)
        self.assertQuerySetEqual(f.qs, ["aaron", "alex"], lambda o: o.username)

        f = F({"favorite_books": ["1", "3"]}, queryset=qs)
        self.assertQuerySetEqual(f.qs, ["aaron", "alex"], lambda o: o.username)

        f = F({"favorite_books": ["2"]}, queryset=qs)
        self.assertQuerySetEqual(f.qs, ["alex"], lambda o: o.username)

        f = F({"favorite_books": ["4"]}, queryset=qs)
        self.assertQuerySetEqual(f.qs, [], lambda o: o.username)

    def test_filtering_on_all_of_subset_of_choices(self):
        class F(FilterSet):
            class Meta:
                model = User
                fields = ["favorite_books"]

            def __init__(self, *args, **kwargs):
                super().__init__(*args, **kwargs)
                # This filter has a limited number of choices.
                self.filters["favorite_books"].extra.update(
                    {"queryset": Book.objects.filter(id__in=[1, 2])}
                )

                self.filters["favorite_books"].extra["required"] = True

        qs = User.objects.all().order_by("username")

        # Select all the given choices.
        f = F({"favorite_books": ["1", "2"]}, queryset=qs)

        # The results should only include matching users - not Jacob.
        self.assertQuerySetEqual(f.qs, ["aaron", "alex"], lambda o: o.username)

    def test_filtering_on_non_required_fields(self):
        # See issue #132 - filtering with all options on a non-required
        # field should exclude any results where the field is null.
        class F(FilterSet):
            author = ModelMultipleChoiceFilter(queryset=User.objects.all())

            class Meta:
                model = Article
                fields = ["author"]

        published = now()
        Article.objects.create(published=published, author=self.alex)
        Article.objects.create(published=published, author=self.alex)
        Article.objects.create(published=published)

        qs = Article.objects.all()

        # Select all authors.
        authors = [str(user.id) for user in User.objects.all()]
        f = F({"author": authors}, queryset=qs)

        # The results should not include anonymous articles
        self.assertEqual(
            set(f.qs),
            set(Article.objects.exclude(author__isnull=True)),
        )


class NumberFilterTests(TestCase):
    def setUp(self):
        Book.objects.create(
            title="Ender's Game", price="10.0", average_rating=4.7999999999999998
        )
        Book.objects.create(
            title="Rainbow Six", price="15.0", average_rating=4.5999999999999996
        )
        Book.objects.create(
            title="Snowcrash", price="20.0", average_rating=4.2999999999999998
        )

    def test_filtering(self):
        class F(FilterSet):
            class Meta:
                model = Book
                fields = ["price"]

        f = F({"price": 10}, queryset=Book.objects.all())
        self.assertQuerySetEqual(f.qs, ["Ender's Game"], lambda o: o.title)


class RangeFilterTests(TestCase):
    def setUp(self):
        Book.objects.create(
            title="Ender's Game", price="10.0", average_rating=4.7999999999999998
        )
        Book.objects.create(
            title="Rainbow Six", price="15.0", average_rating=4.5999999999999996
        )
        Book.objects.create(
            title="Snowcrash", price="20.0", average_rating=4.2999999999999998
        )
        Book.objects.create(title="Refund", price="-10.0", average_rating=5.0)
        Book.objects.create(title="Free Book", price="0.0", average_rating=0.0)

    def test_filtering(self):
        class F(FilterSet):
            price = RangeFilter()

            class Meta:
                model = Book
                fields = ["price"]

        qs = Book.objects.all().order_by("title")
        f = F(queryset=qs)
        self.assertQuerySetEqual(
            f.qs,
            ["Ender's Game", "Free Book", "Rainbow Six", "Refund", "Snowcrash"],
            lambda o: o.title,
        )
        f = F({"price_min": "5", "price_max": "15"}, queryset=qs)
        self.assertQuerySetEqual(
            f.qs, ["Ender's Game", "Rainbow Six"], lambda o: o.title
        )

        f = F({"price_min": "11"}, queryset=qs)
        self.assertQuerySetEqual(f.qs, ["Rainbow Six", "Snowcrash"], lambda o: o.title)
        f = F({"price_max": "19"}, queryset=qs)
        self.assertQuerySetEqual(
            f.qs,
            ["Ender's Game", "Free Book", "Rainbow Six", "Refund"],
            lambda o: o.title,
        )

        f = F({"price_min": "0", "price_max": "12"}, queryset=qs)
        self.assertQuerySetEqual(f.qs, ["Ender's Game", "Free Book"], lambda o: o.title)
        f = F({"price_min": "-11", "price_max": "0"}, queryset=qs)
        self.assertQuerySetEqual(f.qs, ["Free Book", "Refund"], lambda o: o.title)
        f = F({"price_min": "0", "price_max": "0"}, queryset=qs)
        self.assertQuerySetEqual(f.qs, ["Free Book"], lambda o: o.title)


class DateRangeFilterTests(TestCase):
    class CommentFilter(FilterSet):
        date = DateRangeFilter()

        class Meta:
            model = Comment
            fields = ["date"]

    @contextlib.contextmanager
    def relative_to(self, today):
        today = make_aware(today)
        yesterday = today - datetime.timedelta(days=1)
        five_days_ago = today - datetime.timedelta(days=5)
        two_weeks_ago = today - datetime.timedelta(days=14)
        two_months_ago = today - datetime.timedelta(days=62)
        two_years_ago = today - datetime.timedelta(days=800)

        alex = User.objects.create(username="alex")
        time = now().time()
        Comment.objects.create(date=two_weeks_ago, author=alex, time=time)
        Comment.objects.create(date=two_years_ago, author=alex, time=time)
        Comment.objects.create(date=five_days_ago, author=alex, time=time)
        Comment.objects.create(date=today, author=alex, time=time)
        Comment.objects.create(date=yesterday, author=alex, time=time)
        Comment.objects.create(date=two_months_ago, author=alex, time=time)

        with mock.patch("django_filters.filters.now") as mock_now:
            mock_now.return_value = today
            yield

    def test_filtering_for_year(self):
        f = self.CommentFilter({"date": "year"})
        with self.relative_to(datetime.datetime(now().year, 4, 1)):
            self.assertQuerySetEqual(f.qs, [1, 3, 4, 5, 6], lambda o: o.pk, False)

    def test_filtering_for_month(self):
        f = self.CommentFilter({"date": "month"})
        with self.relative_to(datetime.datetime(now().year, 4, 21)):
            self.assertQuerySetEqual(f.qs, [1, 3, 4, 5], lambda o: o.pk, False)

    def test_filtering_for_week(self):
        f = self.CommentFilter({"date": "week"})
        with self.relative_to(datetime.datetime(now().year, 1, 1)):
            self.assertQuerySetEqual(f.qs, [3, 4, 5], lambda o: o.pk, False)

    def test_filtering_for_yesterday(self):
        f = self.CommentFilter({"date": "yesterday"})
        with self.relative_to(datetime.datetime(now().year, 1, 1)):
            self.assertQuerySetEqual(f.qs, [5], lambda o: o.pk, False)

    def test_filtering_for_today(self):
        f = self.CommentFilter({"date": "today"})
        with self.relative_to(datetime.datetime(now().year, 1, 1)):
            self.assertQuerySetEqual(f.qs, [4], lambda o: o.pk, False)


class DateFromToRangeFilterTests(TestCase):
    def test_filtering(self):
        adam = User.objects.create(username="adam")
        kwargs = {"text": "test", "author": adam, "time": "10:00"}
        Comment.objects.create(date=datetime.date(2016, 1, 1), **kwargs)
        Comment.objects.create(date=datetime.date(2016, 1, 2), **kwargs)
        Comment.objects.create(date=datetime.date(2016, 1, 3), **kwargs)
        Comment.objects.create(date=datetime.date(2016, 1, 3), **kwargs)

        class F(FilterSet):
            published = DateFromToRangeFilter(field_name="date")

            class Meta:
                model = Comment
                fields = ["date"]

        results = F(
            data={"published_after": "2016-01-02", "published_before": "2016-01-03"}
        )
        self.assertEqual(len(results.qs), 3)

    def test_filtering_ignores_time(self):
        tz = timezone.get_current_timezone()
        Article.objects.create(
            published=datetime.datetime(2016, 1, 1, 10, 0, tzinfo=tz)
        )
        Article.objects.create(
            published=datetime.datetime(2016, 1, 2, 12, 45, tzinfo=tz)
        )
        Article.objects.create(
            published=datetime.datetime(2016, 1, 3, 18, 15, tzinfo=tz)
        )
        Article.objects.create(
            published=datetime.datetime(2016, 1, 3, 19, 30, tzinfo=tz)
        )

        class F(FilterSet):
            published = DateFromToRangeFilter()

            class Meta:
                model = Article
                fields = ["published"]

        results = F(
            data={"published_after": "2016-01-02", "published_before": "2016-01-03"}
        )
        self.assertEqual(len(results.qs), 3)

    @unittest.skipUnless(django.VERSION < (5, 0), "is_dst removed in Django 5.0")
    @override_settings(TIME_ZONE="America/Sao_Paulo")
    def test_filtering_dst_start_midnight(self):
        tz = timezone.get_default_timezone()
        Article.objects.create(
            published=make_aware(datetime.datetime(2017, 10, 14, 23, 59), tz, False)
        )
        Article.objects.create(
            published=make_aware(datetime.datetime(2017, 10, 15, 0, 0), tz, False)
        )
        Article.objects.create(
            published=make_aware(datetime.datetime(2017, 10, 15, 1, 0), tz, False)
        )
        Article.objects.create(
            published=make_aware(datetime.datetime(2017, 10, 16, 0, 0), tz, False)
        )

        class F(FilterSet):
            published = DateFromToRangeFilter()

            class Meta:
                model = Article
                fields = ["published"]

        results = F(
            data={"published_after": "2017-10-15", "published_before": "2017-10-15"}
        )
        self.assertEqual(len(results.qs), 2)

    @unittest.skipUnless(django.VERSION < (5, 0), "is_dst removed in Django 5.0")
    @override_settings(TIME_ZONE="America/Sao_Paulo")
    def test_filtering_dst_ends_midnight(self):
        tz = timezone.get_default_timezone()
        Article.objects.create(
            published=make_aware(datetime.datetime(2017, 2, 19, 0, 0), tz, False)
        )
        Article.objects.create(
            published=make_aware(datetime.datetime(2017, 2, 18, 23, 0), tz, False)
        )
        Article.objects.create(
            published=make_aware(datetime.datetime(2017, 2, 18, 0, 0), tz, False)
        )
        Article.objects.create(
            published=make_aware(datetime.datetime(2017, 2, 17, 15, 0), tz, False)
        )

        class F(FilterSet):
            published = DateFromToRangeFilter()

            class Meta:
                model = Article
                fields = ["published"]

        results = F(
            data={"published_after": "2017-02-18", "published_before": "2017-02-18"}
        )
        self.assertEqual(len(results.qs), 2)

    @unittest.skipUnless(django.VERSION < (5, 0), "is_dst removed in Django 5.0")
    @override_settings(TIME_ZONE="Europe/Paris")
    def test_filtering_dst_start(self):
        tz = timezone.get_default_timezone()
        Article.objects.create(
            published=make_aware(datetime.datetime(2017, 3, 25, 23, 59), tz, False)
        )
        Article.objects.create(
            published=make_aware(datetime.datetime(2017, 3, 26, 0, 0), tz, False)
        )
        Article.objects.create(
            published=make_aware(datetime.datetime(2017, 3, 26, 2, 0), tz, False)
        )
        Article.objects.create(
            published=make_aware(datetime.datetime(2017, 3, 26, 3, 0), tz, False)
        )
        Article.objects.create(
            published=make_aware(datetime.datetime(2017, 3, 27, 0, 0), tz, False)
        )

        class F(FilterSet):
            published = DateFromToRangeFilter()

            class Meta:
                model = Article
                fields = ["published"]

        results = F(
            data={"published_after": "2017-3-26", "published_before": "2017-3-26"}
        )
        self.assertEqual(len(results.qs), 3)

    @unittest.skipUnless(django.VERSION < (5, 0), "is_dst removed in Django 5.0")
    @override_settings(TIME_ZONE="Europe/Paris")
    def test_filtering_dst_end(self):
        tz = timezone.get_default_timezone()
        Article.objects.create(
            published=make_aware(datetime.datetime(2017, 10, 28, 23, 59), tz, False)
        )
        Article.objects.create(
            published=make_aware(datetime.datetime(2017, 10, 29, 0, 0), tz, False)
        )
        Article.objects.create(
            published=make_aware(datetime.datetime(2017, 10, 29, 2, 0), tz, False)
        )
        Article.objects.create(
            published=make_aware(datetime.datetime(2017, 10, 29, 3, 0), tz, False)
        )
        Article.objects.create(
            published=make_aware(datetime.datetime(2017, 10, 30, 0, 0), tz, False)
        )

        class F(FilterSet):
            published = DateFromToRangeFilter()

            class Meta:
                model = Article
                fields = ["published"]

        results = F(
            data={"published_after": "2017-10-29", "published_before": "2017-10-29"}
        )
        self.assertEqual(len(results.qs), 3)


class DateTimeFromToRangeFilterTests(TestCase):
    def test_filtering(self):
        tz = timezone.get_current_timezone()
        Article.objects.create(
            published=datetime.datetime(2016, 1, 1, 10, 0, tzinfo=tz)
        )
        Article.objects.create(
            published=datetime.datetime(2016, 1, 2, 12, 45, tzinfo=tz)
        )
        Article.objects.create(
            published=datetime.datetime(2016, 1, 3, 18, 15, tzinfo=tz)
        )
        Article.objects.create(
            published=datetime.datetime(2016, 1, 3, 19, 30, tzinfo=tz)
        )

        class F(FilterSet):
            published = DateTimeFromToRangeFilter()

            class Meta:
                model = Article
                fields = ["published"]

        results = F(
            data={
                "published_after": "2016-01-02 10:00",
                "published_before": "2016-01-03 19:00",
            }
        )
        self.assertEqual(len(results.qs), 2)


class IsoDateTimeFromToRangeFilterTests(TestCase):
    def test_filtering(self):
        tz = timezone.get_current_timezone()
        Article.objects.create(
            published=datetime.datetime(2016, 1, 1, 10, 0, tzinfo=tz)
        )
        Article.objects.create(
            published=datetime.datetime(2016, 1, 2, 12, 45, tzinfo=tz)
        )
        Article.objects.create(
            published=datetime.datetime(2016, 1, 3, 18, 15, tzinfo=tz)
        )
        Article.objects.create(
            published=datetime.datetime(2016, 1, 3, 19, 30, tzinfo=tz)
        )

        class F(FilterSet):
            published = IsoDateTimeFromToRangeFilter()

            class Meta:
                model = Article
                fields = ["published"]

        dt = datetime.datetime.now(tz=tz)
        results = F(
            data={
                "published_after": "2016-01-02T10:00:00.000000" + dt.strftime("%z"),
                "published_before": "2016-01-03T19:00:00.000000" + dt.strftime("%z"),
            }
        )
        self.assertEqual(len(results.qs), 2)


class TimeRangeFilterTests(TestCase):
    def test_filtering(self):
        adam = User.objects.create(username="adam")
        kwargs = {"text": "test", "author": adam, "date": datetime.date.today()}
        Comment.objects.create(time="7:30", **kwargs)
        Comment.objects.create(time="8:00", **kwargs)
        Comment.objects.create(time="9:30", **kwargs)
        Comment.objects.create(time="11:00", **kwargs)

        class F(FilterSet):
            time = TimeRangeFilter()

            class Meta:
                model = Comment
                fields = ["time"]

        results = F(data={"time_after": "8:00", "time_before": "10:00"})
        self.assertEqual(len(results.qs), 2)


class AllValuesFilterTests(TestCase):
    def test_filtering(self):
        User.objects.create(username="alex")
        User.objects.create(username="jacob")
        User.objects.create(username="aaron")

        class F(FilterSet):
            username = AllValuesFilter()

            class Meta:
                model = User
                fields = ["username"]

        self.assertEqual(list(F().qs), list(User.objects.all()))
        self.assertEqual(
            list(F({"username": "alex"}).qs), [User.objects.get(username="alex")]
        )

        # invalid choice
        self.assertFalse(F({"username": "jose"}).is_valid())
        self.assertEqual(list(F({"username": "jose"}).qs), list(User.objects.all()))


class AllValuesMultipleFilterTests(TestCase):
    def test_filtering(self):
        User.objects.create(username="alex")
        User.objects.create(username="jacob")
        User.objects.create(username="aaron")

        class F(FilterSet):
            username = AllValuesMultipleFilter()

            class Meta:
                model = User
                fields = ["username"]

        self.assertEqual(list(F().qs), list(User.objects.all()))
        self.assertEqual(
            list(F({"username": ["alex"]}).qs), [User.objects.get(username="alex")]
        )
        self.assertEqual(
            list(F({"username": ["alex", "jacob"]}).qs),
            list(User.objects.filter(username__in=["alex", "jacob"])),
        )

        # invalid choice
        self.assertFalse(F({"username": "jose"}).is_valid())
        self.assertEqual(list(F({"username": "jose"}).qs), list(User.objects.all()))


class FilterMethodTests(TestCase):
    def setUp(self):
        User.objects.create(username="alex")
        User.objects.create(username="jacob")
        User.objects.create(username="aaron")

    def test_filtering(self):
        class F(FilterSet):
            username = CharFilter(method="filter_username")

            class Meta:
                model = User
                fields = ["username"]

            def filter_username(self, queryset, name, value):
                return queryset.filter(**{name: value})

        self.assertEqual(list(F().qs), list(User.objects.all()))
        self.assertEqual(
            list(F({"username": "alex"}).qs), [User.objects.get(username="alex")]
        )
        self.assertEqual(list(F({"username": "jose"}).qs), list())

    def test_filtering_callable(self):
        def filter_username(queryset, name, value):
            return queryset.filter(**{name: value})

        class F(FilterSet):
            username = CharFilter(method=filter_username)

            class Meta:
                model = User
                fields = ["username"]

        self.assertEqual(list(F().qs), list(User.objects.all()))
        self.assertEqual(
            list(F({"username": "alex"}).qs), [User.objects.get(username="alex")]
        )
        self.assertEqual(list(F({"username": "jose"}).qs), list())


class O2ORelationshipTests(TestCase):
    def setUp(self):
        a1 = Account.objects.create(
            name="account1", in_good_standing=False, friendly=False
        )
        a2 = Account.objects.create(
            name="account2", in_good_standing=True, friendly=True
        )
        a3 = Account.objects.create(
            name="account3", in_good_standing=True, friendly=False
        )
        a4 = Account.objects.create(
            name="account4", in_good_standing=False, friendly=True
        )
        Profile.objects.create(account=a1, likes_coffee=True, likes_tea=False)
        Profile.objects.create(account=a2, likes_coffee=False, likes_tea=True)
        Profile.objects.create(account=a3, likes_coffee=True, likes_tea=True)
        Profile.objects.create(account=a4, likes_coffee=False, likes_tea=False)

    def test_o2o_relation(self):
        class F(FilterSet):
            class Meta:
                model = Profile
                fields = ("account",)

        f = F()
        self.assertEqual(f.qs.count(), 4)

        f = F({"account": 1})
        self.assertEqual(f.qs.count(), 1)
        self.assertQuerySetEqual(f.qs, [1], lambda o: o.pk)

    def test_o2o_relation_dictionary(self):
        class F(FilterSet):
            class Meta:
                model = Profile
                fields = {
                    "account": ["exact"],
                }

        f = F()
        self.assertEqual(f.qs.count(), 4)

        f = F({"account": 1})
        self.assertEqual(f.qs.count(), 1)
        self.assertQuerySetEqual(f.qs, [1], lambda o: o.pk)

    def test_reverse_o2o_relation(self):
        class F(FilterSet):
            class Meta:
                model = Account
                fields = ("profile",)

        f = F()
        self.assertEqual(f.qs.count(), 4)

        f = F({"profile": 1})
        self.assertEqual(f.qs.count(), 1)
        self.assertQuerySetEqual(f.qs, [1], lambda o: o.pk)

    def test_o2o_relation_attribute(self):
        class F(FilterSet):
            class Meta:
                model = Profile
                fields = ("account__in_good_standing",)

        f = F()
        self.assertEqual(f.qs.count(), 4)

        f = F({"account__in_good_standing": "2"})
        self.assertEqual(f.qs.count(), 2)
        self.assertQuerySetEqual(f.qs, [2, 3], lambda o: o.pk, False)

    def test_o2o_relation_attribute2(self):
        class F(FilterSet):
            class Meta:
                model = Profile
                fields = (
                    "account__in_good_standing",
                    "account__friendly",
                )

        f = F()
        self.assertEqual(f.qs.count(), 4)

        f = F({"account__in_good_standing": "2", "account__friendly": "2"})
        self.assertEqual(f.qs.count(), 1)
        self.assertQuerySetEqual(f.qs, [2], lambda o: o.pk)

    def test_reverse_o2o_relation_attribute(self):
        class F(FilterSet):
            class Meta:
                model = Account
                fields = ("profile__likes_coffee",)

        f = F()
        self.assertEqual(f.qs.count(), 4)

        f = F({"profile__likes_coffee": "2"})
        self.assertEqual(f.qs.count(), 2)
        self.assertQuerySetEqual(f.qs, [1, 3], lambda o: o.pk, False)

    def test_reverse_o2o_relation_attribute2(self):
        class F(FilterSet):
            class Meta:
                model = Account
                fields = ("profile__likes_coffee", "profile__likes_tea")

        f = F()
        self.assertEqual(f.qs.count(), 4)

        f = F({"profile__likes_coffee": "2", "profile__likes_tea": "2"})
        self.assertEqual(f.qs.count(), 1)
        self.assertQuerySetEqual(f.qs, [3], lambda o: o.pk)


class FKRelationshipTests(TestCase):
    def test_fk_relation(self):
        company1 = Company.objects.create(name="company1")
        company2 = Company.objects.create(name="company2")
        Location.objects.create(company=company1, open_days="some", zip_code="90210")
        Location.objects.create(company=company2, open_days="WEEKEND", zip_code="11111")
        Location.objects.create(company=company1, open_days="monday", zip_code="12345")

        class F(FilterSet):
            class Meta:
                model = Location
                fields = ("company",)

        f = F()
        self.assertEqual(f.qs.count(), 3)

        f = F({"company": 1})
        self.assertEqual(f.qs.count(), 2)
        self.assertQuerySetEqual(f.qs, [1, 3], lambda o: o.pk, False)

    def test_reverse_fk_relation(self):
        alex = User.objects.create(username="alex")
        jacob = User.objects.create(username="jacob")
        date = now().date()
        time = now().time()
        Comment.objects.create(text="comment 1", author=jacob, time=time, date=date)
        Comment.objects.create(text="comment 2", author=alex, time=time, date=date)
        Comment.objects.create(text="comment 3", author=jacob, time=time, date=date)

        class F(FilterSet):
            class Meta:
                model = User
                fields = ["comments"]

        qs = User.objects.all()
        f = F({"comments": [2]}, queryset=qs)
        self.assertQuerySetEqual(f.qs, ["alex"], lambda o: o.username)

        class F(FilterSet):
            comments = AllValuesFilter()

            class Meta:
                model = User
                fields = ["comments"]

        f = F({"comments": 2}, queryset=qs)
        self.assertQuerySetEqual(f.qs, ["alex"], lambda o: o.username)

    def test_fk_relation_attribute(self):
        now_dt = now()
        alex = User.objects.create(username="alex")
        jacob = User.objects.create(username="jacob")
        User.objects.create(username="aaron")

        Article.objects.create(author=alex, published=now_dt)
        Article.objects.create(author=jacob, published=now_dt)
        Article.objects.create(author=alex, published=now_dt)

        class F(FilterSet):
            class Meta:
                model = Article
                fields = ["author__username"]

        self.assertEqual(list(F.base_filters), ["author__username"])
        self.assertEqual(F({"author__username": "alex"}).qs.count(), 2)
        self.assertEqual(F({"author__username": "jacob"}).qs.count(), 1)

        class F(FilterSet):
            author__username = AllValuesFilter()

            class Meta:
                model = Article
                fields = ["author__username"]

        self.assertEqual(F({"author__username": "alex"}).qs.count(), 2)

    def test_reverse_fk_relation_attribute(self):
        alex = User.objects.create(username="alex")
        jacob = User.objects.create(username="jacob")
        date = now().date()
        time = now().time()
        Comment.objects.create(text="comment 1", author=jacob, time=time, date=date)
        Comment.objects.create(text="comment 2", author=alex, time=time, date=date)
        Comment.objects.create(text="comment 3", author=jacob, time=time, date=date)

        class F(FilterSet):
            class Meta:
                model = User
                fields = ["comments__text"]

        qs = User.objects.all()
        f = F({"comments__text": "comment 2"}, queryset=qs)
        self.assertQuerySetEqual(f.qs, ["alex"], lambda o: o.username)

        class F(FilterSet):
            comments__text = AllValuesFilter()

            class Meta:
                model = User
                fields = ["comments__text"]

        f = F({"comments__text": "comment 2"}, queryset=qs)
        self.assertQuerySetEqual(f.qs, ["alex"], lambda o: o.username)

    @unittest.skip("todo - need correct models")
    def test_fk_relation_multiple_attributes(self):
        pass

    @unittest.expectedFailure
    def test_reverse_fk_relation_multiple_attributes(self):
        company = Company.objects.create(name="company")
        Location.objects.create(company=company, open_days="some", zip_code="90210")
        Location.objects.create(company=company, open_days="WEEKEND", zip_code="11111")

        class F(FilterSet):
            class Meta:
                model = Company
                fields = ("locations__zip_code", "locations__open_days")

        f = F({"locations__zip_code": "90210", "locations__open_days": "WEEKEND"})
        self.assertEqual(f.qs.count(), 0)


class M2MRelationshipTests(TestCase):
    def setUp(self):
        alex = User.objects.create(username="alex", status=1)
        User.objects.create(username="jacob", status=1)
        aaron = User.objects.create(username="aaron", status=1)
        b1 = Book.objects.create(title="Ender's Game", price="1.00", average_rating=3.0)
        b2 = Book.objects.create(title="Rainbow Six", price="2.00", average_rating=4.0)
        b3 = Book.objects.create(title="Snowcrash", price="1.00", average_rating=4.0)
        Book.objects.create(
            title="Stranger in a Strage Land", price="2.00", average_rating=3.0
        )
        alex.favorite_books.add(b1, b2)
        aaron.favorite_books.add(b1, b3)

    def test_m2m_relation(self):
        class F(FilterSet):
            class Meta:
                model = User
                fields = ["favorite_books"]

        qs = User.objects.all().order_by("username")
        f = F({"favorite_books": ["1"]}, queryset=qs)
        self.assertQuerySetEqual(f.qs, ["aaron", "alex"], lambda o: o.username)

        f = F({"favorite_books": ["1", "3"]}, queryset=qs)
        self.assertQuerySetEqual(f.qs, ["aaron", "alex"], lambda o: o.username)

        f = F({"favorite_books": ["2"]}, queryset=qs)
        self.assertQuerySetEqual(f.qs, ["alex"], lambda o: o.username)

        f = F({"favorite_books": ["4"]}, queryset=qs)
        self.assertQuerySetEqual(f.qs, [], lambda o: o.username)

    def test_reverse_m2m_relation(self):
        class F(FilterSet):
            class Meta:
                model = Book
                fields = ["lovers"]

        qs = Book.objects.all().order_by("title")
        f = F({"lovers": [1]}, queryset=qs)
        self.assertQuerySetEqual(
            f.qs, ["Ender's Game", "Rainbow Six"], lambda o: o.title
        )

        class F(FilterSet):
            lovers = AllValuesFilter()

            class Meta:
                model = Book
                fields = ["lovers"]

        f = F({"lovers": 1}, queryset=qs)
        self.assertQuerySetEqual(
            f.qs, ["Ender's Game", "Rainbow Six"], lambda o: o.title
        )

    def test_m2m_relation_attribute(self):
        class F(FilterSet):
            class Meta:
                model = User
                fields = ["favorite_books__title"]

        qs = User.objects.all().order_by("username")
        f = F({"favorite_books__title": "Ender's Game"}, queryset=qs)
        self.assertQuerySetEqual(f.qs, ["aaron", "alex"], lambda o: o.username)

        f = F({"favorite_books__title": "Rainbow Six"}, queryset=qs)
        self.assertQuerySetEqual(f.qs, ["alex"], lambda o: o.username)

        # No choices given, so filtering does nothing.
        class F(FilterSet):
            favorite_books__title = MultipleChoiceFilter()

            class Meta:
                model = User
                fields = ["favorite_books__title"]

        f = F()
        self.assertEqual(len(f.filters["favorite_books__title"].field.choices), 0)

        # Specifying choices allows filter to work. (See also AllValues variants.)
        class F(FilterSet):
            favorite_books__title = MultipleChoiceFilter(
                choices=[
                    (b.title, b.title) for b in Book.objects.all()
                ]
            )

            class Meta:
                model = User
                fields = ["favorite_books__title"]

        f = F({'favorite_books__title': ["Ender's Game", "Snowcrash"]}, queryset=qs)
        self.assertIs(True, f.form.is_valid(), list(f.filters["favorite_books__title"].field.choices))
        self.assertQuerySetEqual(
            f.qs, ['aaron', 'alex'],
            lambda o: o.username,
        )

        class F(FilterSet):
            favorite_books__title = AllValuesFilter()

            class Meta:
                model = User
                fields = ["favorite_books__title"]

        f = F({"favorite_books__title": "Snowcrash"}, queryset=qs)
        self.assertQuerySetEqual(f.qs, ["aaron"], lambda o: o.username)

    def test_reverse_m2m_relation_attribute(self):
        class F(FilterSet):
            class Meta:
                model = Book
                fields = ["lovers__username"]

        qs = Book.objects.all().order_by("title")
        f = F({"lovers__username": "alex"}, queryset=qs)
        self.assertQuerySetEqual(
            f.qs, ["Ender's Game", "Rainbow Six"], lambda o: o.title
        )

        f = F({"lovers__username": "jacob"}, queryset=qs)
        self.assertQuerySetEqual(f.qs, [], lambda o: o.title)

        class F(FilterSet):
            lovers__username = MultipleChoiceFilter()

            class Meta:
                model = Book
                fields = ["lovers__username"]

        f = F()
        self.assertEqual(len(f.filters["lovers__username"].field.choices), 0)
        # f = F({'lovers__username': ['1', '3']},
        #     queryset=qs)
        # self.assertQuerySetEqual(
        #     f.qs, ["Ender's Game", "Rainbow Six"], lambda o: o.title)

        class F(FilterSet):
            lovers__username = AllValuesFilter()

            class Meta:
                model = Book
                fields = ["lovers__username"]

        f = F({"lovers__username": "alex"}, queryset=qs)
        self.assertQuerySetEqual(
            f.qs, ["Ender's Game", "Rainbow Six"], lambda o: o.title
        )

    @unittest.expectedFailure
    def test_m2m_relation_multiple_attributes(self):
        class F(FilterSet):
            class Meta:
                model = User
                fields = ["favorite_books__price", "favorite_books__average_rating"]

        qs = User.objects.all().order_by("username")
        f = F(
            {"favorite_books__price": "1.00", "favorite_books__average_rating": 4.0},
            queryset=qs,
        )
        self.assertQuerySetEqual(f.qs, ["aaron"], lambda o: o.username)

        f = F(
            {"favorite_books__price": "3.00", "favorite_books__average_rating": 4.0},
            queryset=qs,
        )
        self.assertQuerySetEqual(f.qs, [], lambda o: o.username)

    @unittest.expectedFailure
    def test_reverse_m2m_relation_multiple_attributes(self):
        class F(FilterSet):
            class Meta:
                model = Book
                fields = ["lovers__status", "lovers__username"]

        qs = Book.objects.all().order_by("title")
        f = F({"lovers__status": 1, "lovers__username": "alex"}, queryset=qs)
        self.assertQuerySetEqual(
            f.qs, ["Ender's Game", "Rainbow Six"], lambda o: o.title
        )

        f = F({"lovers__status": 1, "lovers__username": "jacob"}, queryset=qs)
        self.assertQuerySetEqual(f.qs, [], lambda o: o.title)

    @unittest.skip("todo")
    def test_fk_relation_on_m2m_relation(self):
        pass

    @unittest.skip("todo")
    def test_fk_relation_attribute_on_m2m_relation(self):
        pass


class SymmetricalSelfReferentialRelationshipTests(TestCase):
    def setUp(self):
        n1 = Node.objects.create(name="one")
        n2 = Node.objects.create(name="two")
        n3 = Node.objects.create(name="three")
        n4 = Node.objects.create(name="four")
        n1.adjacents.add(n2)
        n2.adjacents.add(n3)
        n2.adjacents.add(n4)
        n4.adjacents.add(n1)

    def test_relation(self):
        class F(FilterSet):
            class Meta:
                model = Node
                fields = ["adjacents"]

        qs = Node.objects.all().order_by("pk")
        f = F({"adjacents": ["1"]}, queryset=qs)
        self.assertQuerySetEqual(f.qs, [2, 4], lambda o: o.pk)


class NonSymmetricalSelfReferentialRelationshipTests(TestCase):
    def setUp(self):
        n1 = DirectedNode.objects.create(name="one")
        n2 = DirectedNode.objects.create(name="two")
        n3 = DirectedNode.objects.create(name="three")
        n4 = DirectedNode.objects.create(name="four")
        n1.outbound_nodes.add(n2)
        n2.outbound_nodes.add(n3)
        n2.outbound_nodes.add(n4)
        n4.outbound_nodes.add(n1)

    def test_forward_relation(self):
        class F(FilterSet):
            class Meta:
                model = DirectedNode
                fields = ["outbound_nodes"]

        qs = DirectedNode.objects.all().order_by("pk")
        f = F({"outbound_nodes": ["1"]}, queryset=qs)
        self.assertQuerySetEqual(f.qs, [4], lambda o: o.pk)

    def test_reverse_relation(self):
        class F(FilterSet):
            class Meta:
                model = DirectedNode
                fields = ["inbound_nodes"]

        qs = DirectedNode.objects.all().order_by("pk")
        f = F({"inbound_nodes": ["1"]}, queryset=qs)
        self.assertQuerySetEqual(f.qs, [2], lambda o: o.pk)


@override_settings(TIME_ZONE="UTC")
class TransformedQueryExpressionFilterTests(TestCase):
    def test_filtering(self):
        now_dt = now()
        after_5pm = now_dt.replace(hour=18)
        before_5pm = now_dt.replace(hour=16)

        u = User.objects.create(username="alex")
        a = Article.objects.create(author=u, published=after_5pm)
        Article.objects.create(author=u, published=before_5pm)

        class F(FilterSet):
            class Meta:
                model = Article
                fields = {"published": ["hour__gte"]}

        qs = Article.objects.all()
        f = F({"published__hour__gte": 17}, queryset=qs)
        self.assertEqual(len(f.qs), 1)
        self.assertQuerySetEqual(f.qs, [a.pk], lambda o: o.pk)


class LookupChoiceFilterTests(TestCase):
    class BookFilter(FilterSet):
        price = LookupChoiceFilter(
            lookup_choices=["lt", "gt"], field_class=forms.DecimalField
        )

        class Meta:
            model = Book
            fields = ["price"]

    @classmethod
    def setUpTestData(cls):
        Book.objects.create(
            title="Ender's Game", price="10.0", average_rating=4.7999999999999998
        )
        Book.objects.create(
            title="Rainbow Six", price="15.0", average_rating=4.5999999999999996
        )
        Book.objects.create(
            title="Snowcrash", price="20.0", average_rating=4.2999999999999998
        )

    def test_filtering(self):
        F = self.BookFilter

        f = F({"price": "15", "price_lookup": "lt"})
        self.assertQuerySetEqual(f.qs, ["Ender's Game"], lambda o: o.title)
        f = F({"price": "15", "price_lookup": "lt"})
        self.assertQuerySetEqual(f.qs, ["Ender's Game"], lambda o: o.title)
        f = F({"price": "", "price_lookup": "lt"})
        self.assertTrue(f.is_valid())
        self.assertQuerySetEqual(
            f.qs,
            ["Ender's Game", "Rainbow Six", "Snowcrash"],
            lambda o: o.title,
            ordered=False,
        )
        f = F({"price": "15"})
        self.assertFalse(f.is_valid())
        self.assertQuerySetEqual(
            f.qs,
            ["Ender's Game", "Rainbow Six", "Snowcrash"],
            lambda o: o.title,
            ordered=False,
        )

    def test_inner_field_class_validation(self):
        f = self.BookFilter({"price": "asdf", "price_lookup": "lt"})
        self.assertFalse(f.is_valid())
        self.assertEqual(
            f.errors,
            {
                "price": ["Enter a number."],
            },
        )

    def test_lookup_choices_validation(self):
        f = self.BookFilter({"price": "1", "price_lookup": "asdf"})
        self.assertFalse(f.is_valid())
        self.assertEqual(
            f.errors,
            {
                "price": [
                    "Select a valid choice. asdf is not one of the available choices."
                ],
            },
        )

    def test_lookup_omitted(self):
        f = self.BookFilter({"price": "1"})
        self.assertFalse(f.is_valid())
        self.assertEqual(
            f.errors,
            {
                "price": ["Select a lookup."],
            },
        )


@override_settings(TIME_ZONE="UTC")
class CSVFilterTests(TestCase):
    def setUp(self):
        u1 = User.objects.create(username="alex", status=1)
        u2 = User.objects.create(username="jacob", status=2)
        User.objects.create(username="aaron", status=2)
        User.objects.create(username="carl", status=0)

        now_dt = now()
        after_5pm = now_dt.replace(hour=18)
        before_5pm = now_dt.replace(hour=16)

        Article.objects.create(author=u1, published=after_5pm)
        Article.objects.create(author=u2, published=after_5pm)
        Article.objects.create(author=u1, published=before_5pm)
        Article.objects.create(author=u2, published=before_5pm)

        class UserFilter(FilterSet):
            class Meta:
                model = User
                fields = {
                    "username": ["in"],
                    "status": ["in"],
                }

        class ArticleFilter(FilterSet):
            class Meta:
                model = Article
                fields = {
                    "author": ["in"],
                    "published": ["in"],
                }

        self.user_filter = UserFilter
        self.article_filter = ArticleFilter

        self.after_5pm = after_5pm.strftime("%Y-%m-%d %H:%M:%S.%f")
        self.before_5pm = before_5pm.strftime("%Y-%m-%d %H:%M:%S.%f")

    def test_numeric_filtering(self):
        F = self.user_filter
        qs = User.objects.order_by("pk")

        cases = [
            (None, [1, 2, 3, 4]),
            (QueryDict("status__in=1&status__in=2"), [2, 3]),
            ({"status__in": ""}, [1, 2, 3, 4]),
            ({"status__in": ","}, []),
            ({"status__in": "0"}, [4]),
            ({"status__in": "0,2"}, [2, 3, 4]),
            ({"status__in": "0,,1"}, [1, 4]),
            ({"status__in": "2"}, [2, 3]),
        ]

        for params, expected in cases:
            with self.subTest(params=params, expected=expected):
                self.assertQuerySetEqual(
                    F(params, queryset=qs).qs, expected, attrgetter("pk")
                )

    def test_string_filtering(self):
        F = self.user_filter
        qs = User.objects.order_by("pk")

        cases = [
            (None, [1, 2, 3, 4]),
            (QueryDict("username__in=alex&username__in=aaron"), [3]),
            ({"username__in": ""}, [1, 2, 3, 4]),
            ({"username__in": ","}, []),
            ({"username__in": "alex"}, [1]),
            ({"username__in": "alex,aaron"}, [1, 3]),
            ({"username__in": "alex,,aaron"}, [1, 3]),
            ({"username__in": "alex,"}, [1]),
        ]

        for params, expected in cases:
            with self.subTest(params=params, expected=expected):
                self.assertQuerySetEqual(
                    F(params, queryset=qs).qs, expected, attrgetter("pk")
                )

    def test_datetime_filtering(self):
        F = self.article_filter
        qs = Article.objects.order_by("pk")

        after = self.after_5pm
        before = self.before_5pm

        cases = [
            (None, [1, 2, 3, 4]),
            (QueryDict("published__in=%s&published__in=%s" % (after, before)), [3, 4]),
            ({"published__in": ""}, [1, 2, 3, 4]),
            ({"published__in": ","}, []),
            ({"published__in": "%s" % (after,)}, [1, 2]),
            (
                {
                    "published__in": "%s,%s"
                    % (
                        after,
                        before,
                    )
                },
                [1, 2, 3, 4],
            ),
            (
                {
                    "published__in": "%s,,%s"
                    % (
                        after,
                        before,
                    )
                },
                [1, 2, 3, 4],
            ),
            ({"published__in": "%s," % (after,)}, [1, 2]),
        ]

        for params, expected in cases:
            with self.subTest(params=params, expected=expected):
                self.assertQuerySetEqual(
                    F(params, queryset=qs).qs, expected, attrgetter("pk")
                )

    def test_related_filtering(self):
        F = self.article_filter
        qs = Article.objects.order_by("pk")

        cases = [
            (None, [1, 2, 3, 4]),
            (QueryDict("author__in=1&author__in=2"), [2, 4]),
            ({"author__in": ""}, [1, 2, 3, 4]),
            ({"author__in": ","}, []),
            ({"author__in": "1"}, [1, 3]),
            ({"author__in": "1,2"}, [1, 2, 3, 4]),
            ({"author__in": "1,,2"}, [1, 2, 3, 4]),
            ({"author__in": "1,"}, [1, 3]),
        ]

        for params, expected in cases:
            with self.subTest(params=params, expected=expected):
                self.assertQuerySetEqual(
                    F(params, queryset=qs).qs, expected, attrgetter("pk")
                )


@override_settings(TIME_ZONE="UTC")
class CSVRangeFilterTests(TestCase):
    class ArticleFilter(FilterSet):
        class Meta:
            model = Article
            fields = {
                "published": ["range"],
            }

    @classmethod
    def setUpTestData(cls):
        u1 = User.objects.create(username="alex", status=1)
        u2 = User.objects.create(username="jacob", status=2)
        User.objects.create(username="aaron", status=2)
        User.objects.create(username="carl", status=0)

        now_dt = now()
        after_5pm = now_dt.replace(hour=18)
        around_5pm = now_dt.replace(hour=17)
        before_5pm = now_dt.replace(hour=16)

        Article.objects.create(author=u1, published=after_5pm)
        Article.objects.create(author=u2, published=around_5pm)
        Article.objects.create(author=u1, published=around_5pm)
        Article.objects.create(author=u2, published=before_5pm)

        cls.after_5pm = after_5pm.strftime("%Y-%m-%d %H:%M:%S.%f")
        cls.around_5pm = around_5pm.strftime("%Y-%m-%d %H:%M:%S.%f")
        cls.before_5pm = before_5pm.strftime("%Y-%m-%d %H:%M:%S.%f")

    def test_filtering(self):
        F = self.ArticleFilter

        f = F()
        self.assertEqual(f.qs.count(), 4)

        # empty value is a noop
        f = F({"published__range": ""})
        self.assertTrue(f.is_valid())
        self.assertEqual(f.qs.count(), 4)

        # empty values are interpreted as None types
        f = F({"published__range": ","})
        self.assertEqual(f.qs.count(), 0)

        f = F({"published__range": "%s" % (self.before_5pm,)})
        self.assertFalse(f.is_valid())

        f = F(
            {
                "published__range": "%s,%s"
                % (
                    self.before_5pm,
                    self.around_5pm,
                )
            }
        )
        self.assertEqual(f.qs.count(), 3)

        f = F(
            {
                "published__range": "%s,,%s"
                % (
                    self.before_5pm,
                    self.after_5pm,
                )
            }
        )
        self.assertFalse(f.is_valid())

        # empty value is interpreted as None type
        f = F({"published__range": "%s," % (self.before_5pm,)})
        self.assertEqual(f.qs.count(), 0)


class OrderingFilterTests(TestCase):
    def setUp(self):
        User.objects.create(username="alex", status=1)
        User.objects.create(username="jacob", status=2)
        User.objects.create(username="aaron", status=2)
        User.objects.create(username="carl", status=0)

    def test_ordering(self):
        class F(FilterSet):
            o = OrderingFilter(fields=("username",))

            class Meta:
                model = User
                fields = ["username"]

        qs = User.objects.all()
        tests = [
            {"o": "username"},
            QueryDict("o=username,"),
        ]
        for data in tests:
            with self.subTest(data=data):
                f = F(data, queryset=qs)
                names = f.qs.values_list("username", flat=True)
                self.assertEqual(list(names), ["aaron", "alex", "carl", "jacob"])

    def test_ordering_with_select_widget(self):
        class F(FilterSet):
            o = OrderingFilter(widget=forms.Select, fields=("username",))

            class Meta:
                model = User
                fields = ["username"]

        qs = User.objects.all()
        f = F({"o": "username"}, queryset=qs)
        names = f.qs.values_list("username", flat=True)
        self.assertEqual(list(names), ["aaron", "alex", "carl", "jacob"])

    def test_csv_input(self):
        class F(FilterSet):
            o = OrderingFilter(widget=forms.Select, fields=("username",),)

            class Meta:
                model = User
                fields = ["username"]

        qs = User.objects.all()
        tests = [
            {"o": ","},
            QueryDict("o=%2c"),
            QueryDict("o=,"),
        ]
        for data in tests:
            with self.subTest(data=data):
                f = F(data, queryset=qs)
                self.assertIs(True, f.is_valid())
                names = f.qs.values_list("username", flat=True)
                self.assertEqual(list(names), ['alex', 'jacob', 'aaron', 'carl'])


class MiscFilterSetTests(TestCase):
    def setUp(self):
        User.objects.create(username="alex", last_name="johnson", status=1)
        User.objects.create(username="jacob", last_name="johnson", status=2)
        User.objects.create(username="aaron", last_name="white", status=2)
        User.objects.create(username="carl", last_name="black", status=0)

    def test_filtering_with_declared_filters(self):
        class F(FilterSet):
            account = CharFilter(field_name="username")

            class Meta:
                model = User
                fields = ["account"]

        qs = MockQuerySet()
        F({"account": "jdoe"}, queryset=qs).qs
        qs.all.return_value.filter.assert_called_with(username__exact="jdoe")

    def test_filtering_without_meta(self):
        class F(FilterSet):
            username = CharFilter()

        f = F({"username": "alex"}, queryset=User.objects.all())
        self.assertQuerySetEqual(f.qs, ["alex"], lambda o: o.username)

    def test_filtering_with_multiple_filters(self):
        class F(FilterSet):
            class Meta:
                model = User
                fields = ["status", "username"]

        qs = User.objects.all()

        f = F({"username": "alex", "status": "1"}, queryset=qs)
        self.assertQuerySetEqual(f.qs, ["alex"], lambda o: o.username)

        f = F({"username": "alex", "status": "2"}, queryset=qs)
        self.assertQuerySetEqual(f.qs, [], lambda o: o.pk)

    def test_filter_with_initial(self):
        # Initial values are a form presentation option - the FilterSet should
        # not use an initial value as a default value to filter by.
        class F(FilterSet):
            status = ChoiceFilter(choices=STATUS_CHOICES, initial=1)

            class Meta:
                model = User
                fields = ["status"]

        qs = User.objects.all()
        users = ["alex", "jacob", "aaron", "carl"]

        f = F(queryset=qs)
        self.assertQuerySetEqual(f.qs.order_by("pk"), users, lambda o: o.username)

        f = F({"status": 0}, queryset=qs)
        self.assertQuerySetEqual(f.qs, ["carl"], lambda o: o.username)

    def test_qs_count(self):
        class F(FilterSet):
            class Meta:
                model = User
                fields = ["status"]

        qs = User.objects.all()
        f = F(queryset=qs)
        self.assertEqual(len(f.qs), 4)
        self.assertEqual(f.qs.count(), 4)

        f = F({"status": "0"}, queryset=qs)
        self.assertEqual(len(f.qs), 1)
        self.assertEqual(f.qs.count(), 1)

        f = F({"status": "1"}, queryset=qs)
        self.assertEqual(len(f.qs), 1)
        self.assertEqual(f.qs.count(), 1)

        f = F({"status": "2"}, queryset=qs)
        self.assertEqual(len(f.qs), 2)
        self.assertEqual(f.qs.count(), 2)

    def test_filtering_with_widgets(self):
        class CharInFilter(BaseInFilter, CharFilter):
            pass

        class F(FilterSet):
            last_name = CharInFilter(widget=QueryArrayWidget)
            username = CharInFilter()

            class Meta:
                model = User
                fields = ["last_name", "username"]

        qs = User.objects.all()

        f = F({"last_name": ["johnson"]}, queryset=qs)
        self.assertQuerySetEqual(
            f.qs, ["alex", "jacob"], lambda o: o.username, ordered=False
        )

        f = F({"last_name": ["johnson"], "username": "carl"}, queryset=qs)
        self.assertQuerySetEqual(f.qs, [], lambda o: o.username, ordered=False)

        f = F({"last_name": ["johnson"], "username": "jacob"}, queryset=qs)
        self.assertQuerySetEqual(f.qs, ["jacob"], lambda o: o.username, ordered=False)

        f = F(
            {"last_name": ["johnson", "white"], "username": "jacob, carl, aaron"},
            queryset=qs,
        )
        self.assertQuerySetEqual(
            f.qs, ["jacob", "aaron"], lambda o: o.username, ordered=False
        )



from django.urls import path

from django_filters.views import FilterView, object_filter

from .models import Book


def _foo():
    return "bar"


urlpatterns = [
    path(
        "books-legacy/",
        object_filter,
        {"model": Book, "extra_context": {"foo": _foo, "bar": "foo"}},
    ),
    path("books/", FilterView.as_view(model=Book)),
]



import datetime
from decimal import Decimal

from django.test import TestCase
from django.test.utils import override_settings
from django.urls import path, reverse
from django.utils.dateparse import parse_date
from rest_framework import generics, serializers, status
from rest_framework.test import APIRequestFactory

from django_filters import filters
from django_filters.rest_framework import DjangoFilterBackend, FilterSet

from .models import (
    BaseFilterableItem,
    BasicModel,
    DjangoFilterOrderingModel,
    FilterableItem,
)

factory = APIRequestFactory()


class FilterableItemSerializer(serializers.ModelSerializer):
    class Meta:
        model = FilterableItem
        fields = "__all__"


# Basic filter on a list view.
class FilterFieldsRootView(generics.ListCreateAPIView):
    queryset = FilterableItem.objects.all()
    serializer_class = FilterableItemSerializer
    filterset_fields = ["decimal", "date"]
    filter_backends = (DjangoFilterBackend,)


# These class are used to test a filter class.
class SeveralFieldsFilter(FilterSet):
    text = filters.CharFilter(lookup_expr="icontains")
    decimal = filters.NumberFilter(lookup_expr="lt")
    date = filters.DateFilter(lookup_expr="gt")

    class Meta:
        model = FilterableItem
        fields = ["text", "decimal", "date"]


class FilterClassRootView(generics.ListCreateAPIView):
    queryset = FilterableItem.objects.all()
    serializer_class = FilterableItemSerializer
    filterset_class = SeveralFieldsFilter
    filter_backends = (DjangoFilterBackend,)


# These classes are used to test a misconfigured filter class.
class MisconfiguredFilter(FilterSet):
    text = filters.CharFilter(lookup_expr="icontains")

    class Meta:
        model = BasicModel
        fields = ["text"]


class IncorrectlyConfiguredRootView(generics.ListCreateAPIView):
    queryset = FilterableItem.objects.all()
    serializer_class = FilterableItemSerializer
    filterset_class = MisconfiguredFilter
    filter_backends = (DjangoFilterBackend,)


class FilterClassDetailView(generics.RetrieveAPIView):
    queryset = FilterableItem.objects.all()
    serializer_class = FilterableItemSerializer
    filterset_class = SeveralFieldsFilter
    filter_backends = (DjangoFilterBackend,)


# These classes are used to test base model filter support
class BaseFilterableItemFilter(FilterSet):
    text = filters.CharFilter()

    class Meta:
        model = BaseFilterableItem
        fields = "__all__"


class BaseFilterableItemFilterRootView(generics.ListCreateAPIView):
    queryset = FilterableItem.objects.all()
    serializer_class = FilterableItemSerializer
    filterset_class = BaseFilterableItemFilter
    filter_backends = (DjangoFilterBackend,)


# Regression test for #814
class FilterFieldsQuerysetView(generics.ListCreateAPIView):
    queryset = FilterableItem.objects.all()
    serializer_class = FilterableItemSerializer
    filterset_fields = ["decimal", "date"]
    filter_backends = (DjangoFilterBackend,)


class GetQuerysetView(generics.ListCreateAPIView):
    serializer_class = FilterableItemSerializer
    filterset_class = SeveralFieldsFilter
    filter_backends = (DjangoFilterBackend,)

    def get_queryset(self):
        return FilterableItem.objects.all()


urlpatterns = [
    path("<int:pk>/", FilterClassDetailView.as_view(), name="detail-view"),
    path("", FilterClassRootView.as_view(), name="root-view"),
    path("get-queryset/", GetQuerysetView.as_view(), name="get-queryset-view"),
]


class CommonFilteringTestCase(TestCase):
    def _serialize_object(self, obj):
        return {
            "id": obj.id,
            "text": obj.text,
            "decimal": str(obj.decimal),
            "date": obj.date.isoformat(),
        }

    def setUp(self):
        """
        Create 10 FilterableItem instances.
        """
        base_data = ("a", Decimal("0.25"), datetime.date(2012, 10, 8))
        for i in range(10):
            text = chr(i + ord(base_data[0])) * 3  # Produces string 'aaa', 'bbb', etc.
            decimal = base_data[1] + i
            date = base_data[2] - datetime.timedelta(days=i * 2)
            FilterableItem(text=text, decimal=decimal, date=date).save()

        self.objects = FilterableItem.objects
        self.data = [self._serialize_object(obj) for obj in self.objects.all()]


class IntegrationTestFiltering(CommonFilteringTestCase):
    """
    Integration tests for filtered list views.
    """

    def test_get_filtered_fields_root_view(self):
        """
        GET requests to paginated ListCreateAPIView should return paginated results.
        """
        view = FilterFieldsRootView.as_view()

        # Basic test with no filter.
        request = factory.get("/")
        response = view(request).render()
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(response.data, self.data)

        # Tests that the decimal filter works.
        search_decimal = Decimal("2.25")
        request = factory.get("/", {"decimal": "%s" % search_decimal})
        response = view(request).render()
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        expected_data = [
            f for f in self.data if Decimal(f["decimal"]) == search_decimal
        ]
        self.assertEqual(response.data, expected_data)

        # Tests that the date filter works.
        search_date = datetime.date(2012, 9, 22)
        request = factory.get(
            "/", {"date": "%s" % search_date}
        )  # search_date str: '2012-09-22'
        response = view(request).render()
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        expected_data = [f for f in self.data if parse_date(f["date"]) == search_date]
        self.assertEqual(response.data, expected_data)

    def test_filter_with_queryset(self):
        """
        Regression test for #814.
        """
        view = FilterFieldsQuerysetView.as_view()

        # Tests that the decimal filter works.
        search_decimal = Decimal("2.25")
        request = factory.get("/", {"decimal": "%s" % search_decimal})
        response = view(request).render()
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        expected_data = [
            f for f in self.data if Decimal(f["decimal"]) == search_decimal
        ]
        self.assertEqual(response.data, expected_data)

    def test_filter_with_get_queryset_only(self):
        """
        Regression test for #834.
        """
        view = GetQuerysetView.as_view()
        request = factory.get("/get-queryset/")
        view(request).render()
        # Used to raise "issubclass() arg 2 must be a class or tuple of classes"
        # here when neither `model' nor `queryset' was specified.

    def test_get_filtered_class_root_view(self):
        """
        GET requests to filtered ListCreateAPIView that have a filterset_class set
        should return filtered results.
        """
        view = FilterClassRootView.as_view()

        # Basic test with no filter.
        request = factory.get("/")
        response = view(request).render()
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(response.data, self.data)

        # Tests that the decimal filter set with 'lt' in the filter class works.
        search_decimal = Decimal("4.25")
        request = factory.get("/", {"decimal": "%s" % search_decimal})
        response = view(request).render()
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        expected_data = [f for f in self.data if Decimal(f["decimal"]) < search_decimal]
        self.assertEqual(response.data, expected_data)

        # Tests that the date filter set with 'gt' in the filter class works.
        search_date = datetime.date(2012, 10, 2)
        request = factory.get(
            "/", {"date": "%s" % search_date}
        )  # search_date str: '2012-10-02'
        response = view(request).render()
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        expected_data = [f for f in self.data if parse_date(f["date"]) > search_date]
        self.assertEqual(response.data, expected_data)

        # Tests that the text filter set with 'icontains' in the filter class works.
        search_text = "ff"
        request = factory.get("/", {"text": "%s" % search_text})
        response = view(request).render()
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        expected_data = [f for f in self.data if search_text in f["text"].lower()]
        self.assertEqual(response.data, expected_data)

        # Tests that multiple filters works.
        search_decimal = Decimal("5.25")
        search_date = datetime.date(2012, 10, 2)
        request = factory.get(
            "/", {"decimal": "%s" % (search_decimal,), "date": "%s" % (search_date,)}
        )
        response = view(request).render()
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        expected_data = [
            f
            for f in self.data
            if parse_date(f["date"]) > search_date
            and Decimal(f["decimal"]) < search_decimal
        ]
        self.assertEqual(response.data, expected_data)

    def test_incorrectly_configured_filter(self):
        """
        An error should be displayed when the filter class is misconfigured.
        """
        view = IncorrectlyConfiguredRootView.as_view()

        request = factory.get("/")
        self.assertRaises(AssertionError, view, request)

    def test_base_model_filter(self):
        """
        The `get_filterset_class` model checks should allow base model filters.
        """
        view = BaseFilterableItemFilterRootView.as_view()

        request = factory.get("/?text=aaa")
        response = view(request).render()
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(len(response.data), 1)

    def test_unknown_filter(self):
        """
        GET requests with filters that aren't configured should return 200.
        """
        view = FilterFieldsRootView.as_view()

        search_integer = 10
        request = factory.get("/", {"integer": "%s" % search_integer})
        response = view(request).render()
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    def test_html_rendering(self):
        """
        Make sure response renders w/ backend
        """
        view = FilterFieldsRootView.as_view()
        request = factory.get("/")
        request.META["HTTP_ACCEPT"] = "text/html"
        response = view(request).render()
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    def test_raise_validation_error(self):
        """
        Ensure validation errors return a proper error response instead of
        an internal server error.
        """
        view = FilterFieldsRootView.as_view()
        request = factory.get("/?decimal=foobar")
        response = view(request).render()
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
        self.assertEqual(response.data, {"decimal": ["Enter a number."]})

    def test_permissive(self):
        """
        Permissive handling should return a partially filtered result set.
        """
        FilterableItem.objects.create(decimal=Decimal("1.23"), date="2017-01-01")
        FilterableItem.objects.create(decimal=Decimal("1.23"), date="2016-01-01")

        class Backend(DjangoFilterBackend):
            raise_exception = False

        class View(FilterFieldsRootView):
            filter_backends = (Backend,)

        view = View.as_view()
        request = factory.get("/?decimal=foobar&date=2017-01-01")
        response = view(request).render()
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(response.data[0]["date"], "2017-01-01")
        self.assertEqual(len(response.data), 1)


@override_settings(ROOT_URLCONF="tests.rest_framework.test_integration")
class IntegrationTestDetailFiltering(CommonFilteringTestCase):
    """
    Integration tests for filtered detail views.
    """

    def _get_url(self, item):
        return reverse("detail-view", kwargs=dict(pk=item.pk))

    def test_get_filtered_detail_view(self):
        """
        GET requests to filtered RetrieveAPIView that have a filterset_class
        set should return filtered results.
        """
        item = self.objects.all()[0]
        data = self._serialize_object(item)

        # Basic test with no filter.
        response = self.client.get(self._get_url(item))
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(response.data, data)

        # Tests that the decimal filter set that should fail.
        search_decimal = Decimal("4.25")
        high_item = self.objects.filter(decimal__gt=search_decimal)[0]
        response = self.client.get(
            "{url}".format(url=self._get_url(high_item)),
            {"decimal": "{param}".format(param=search_decimal)},
        )
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        # Tests that the decimal filter set that should succeed.
        search_decimal = Decimal("4.25")
        low_item = self.objects.filter(decimal__lt=search_decimal)[0]
        low_item_data = self._serialize_object(low_item)
        response = self.client.get(
            "{url}".format(url=self._get_url(low_item)),
            {"decimal": "{param}".format(param=search_decimal)},
        )
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(response.data, low_item_data)

        # Tests that multiple filters works.
        search_decimal = Decimal("5.25")
        search_date = datetime.date(2012, 10, 2)
        valid_item = self.objects.filter(
            decimal__lt=search_decimal, date__gt=search_date
        )[0]
        valid_item_data = self._serialize_object(valid_item)
        response = self.client.get(
            "{url}".format(url=self._get_url(valid_item)),
            {
                "decimal": "{decimal}".format(decimal=search_decimal),
                "date": "{date}".format(date=search_date),
            },
        )
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(response.data, valid_item_data)


class DjangoFilterOrderingSerializer(serializers.ModelSerializer):
    class Meta:
        model = DjangoFilterOrderingModel
        fields = "__all__"


class DjangoFilterOrderingTests(TestCase):
    def setUp(self):
        data = [
            {"date": datetime.date(2012, 10, 8), "text": "abc"},
            {"date": datetime.date(2013, 10, 8), "text": "bcd"},
            {"date": datetime.date(2014, 10, 8), "text": "cde"},
        ]

        for d in data:
            DjangoFilterOrderingModel.objects.create(**d)

    def test_default_ordering(self):
        class DjangoFilterOrderingView(generics.ListAPIView):
            serializer_class = DjangoFilterOrderingSerializer
            queryset = DjangoFilterOrderingModel.objects.all()
            filter_backends = (DjangoFilterBackend,)
            filterset_fields = ["text"]
            ordering = ("-date",)

        view = DjangoFilterOrderingView.as_view()
        request = factory.get("/")
        response = view(request)

        self.assertEqual(
            response.data,
            [
                {"id": 3, "date": "2014-10-08", "text": "cde"},
                {"id": 2, "date": "2013-10-08", "text": "bcd"},
                {"id": 1, "date": "2012-10-08", "text": "abc"},
            ],
        )



from unittest import mock

from django.test import TestCase, override_settings

from django_filters.conf import is_callable, settings


class DefaultSettingsTests(TestCase):
    def test_verbose_lookups(self):
        self.assertIsInstance(settings.VERBOSE_LOOKUPS, dict)
        self.assertIn("exact", settings.VERBOSE_LOOKUPS)

    def test_default_lookup_expr(self):
        self.assertEqual(settings.DEFAULT_LOOKUP_EXPR, "exact")

    def test_disable_help_text(self):
        self.assertFalse(settings.DISABLE_HELP_TEXT)

    def test_empty_choice_label(self):
        self.assertEqual(settings.EMPTY_CHOICE_LABEL, "---------")

    def test_null_choice_label(self):
        self.assertIsNone(settings.NULL_CHOICE_LABEL)

    def test_null_choice_value(self):
        self.assertEqual(settings.NULL_CHOICE_VALUE, "null")


class OverrideSettingsTests(TestCase):
    def test_attribute_override(self):
        self.assertIsInstance(settings.VERBOSE_LOOKUPS, dict)

        original = settings.VERBOSE_LOOKUPS

        with override_settings(FILTERS_VERBOSE_LOOKUPS=None):
            self.assertIsNone(settings.VERBOSE_LOOKUPS)

        self.assertIs(settings.VERBOSE_LOOKUPS, original)

    def test_missing_attribute_override(self):
        # ensure that changed setting behaves correctly when
        # not originally present in the user's settings.
        from django.conf import settings as dj_settings

        self.assertFalse(hasattr(dj_settings, "FILTERS_DISABLE_HELP_TEXT"))

        # Default value
        self.assertFalse(settings.DISABLE_HELP_TEXT)

        with override_settings(FILTERS_DISABLE_HELP_TEXT=True):
            self.assertTrue(settings.DISABLE_HELP_TEXT)

        # Revert to default
        self.assertFalse(settings.DISABLE_HELP_TEXT)

    def test_non_filters_setting(self):
        self.assertFalse(hasattr(settings, "USE_TZ"))

        with override_settings(USE_TZ=False):
            self.assertFalse(hasattr(settings, "USE_TZ"))

        self.assertFalse(hasattr(settings, "USE_TZ"))

    def test_non_existent_setting(self):
        self.assertFalse(hasattr(settings, "FILTERS_FOOBAR"))
        self.assertFalse(hasattr(settings, "FOOBAR"))

        with override_settings(FILTERS_FOOBAR="blah"):
            self.assertFalse(hasattr(settings, "FILTERS_FOOBAR"))
            self.assertFalse(hasattr(settings, "FOOBAR"))

        self.assertFalse(hasattr(settings, "FILTERS_FOOBAR"))
        self.assertFalse(hasattr(settings, "FOOBAR"))


class IsCallableTests(TestCase):
    def test_behavior(self):
        def func():
            pass

        class Class:
            def __call__(self):
                pass

            def method(self):
                pass

        c = Class()

        self.assertTrue(is_callable(func))
        self.assertFalse(is_callable(Class))
        self.assertTrue(is_callable(c))
        self.assertTrue(is_callable(c.method))


class SettingsObjectTestCase(TestCase):
    @mock.patch("django_filters.conf.DEPRECATED_SETTINGS", ["TEST_123"])
    @mock.patch.dict("django_filters.conf.DEFAULTS", {"TEST_123": True})
    def test_get_setting_deprecated(self):
        with override_settings(FILTERS_TEST_123=True):
            with self.assertWarns(DeprecationWarning):
                settings.change_setting("FILTERS_TEST_123", True, True)
                test_setting = settings.get_setting("TEST_123")
                self.assertTrue(test_setting)



from django import forms
from django.db import models
from django.utils.translation import gettext_lazy as _

REGULAR = 0
MANAGER = 1
ADMIN = 2

STATUS_CHOICES = (
    (REGULAR, "Regular"),
    (MANAGER, "Manager"),
    (ADMIN, "Admin"),
)


# classes for testing filters with inherited fields
class SubCharField(models.CharField):
    pass


class SubSubCharField(SubCharField):
    pass


class SubnetMaskField(models.Field):
    empty_strings_allowed = False
    description = "Subnet Mask"

    def __init__(self, *args, **kwargs):
        kwargs["max_length"] = 15
        models.Field.__init__(self, *args, **kwargs)

    def get_internal_type(self):
        return "GenericIPAddressField"

    def formfield(self, **kwargs):
        defaults = {"form_class": forms.GenericIPAddressField}
        defaults.update(kwargs)
        return super().formfield(**defaults)


class User(models.Model):
    username = models.CharField(_("username"), max_length=255)
    first_name = SubCharField(max_length=100)
    last_name = SubSubCharField(max_length=100)

    status = models.IntegerField(choices=STATUS_CHOICES, default=0)

    is_active = models.BooleanField(default=False)
    is_employed = models.BooleanField(null=True, default=False)

    favorite_books = models.ManyToManyField("Book", related_name="lovers")

    def __str__(self):
        return self.username


class ManagerGroup(models.Model):
    users = models.ManyToManyField(
        User, limit_choices_to={"is_active": True}, related_name="member_of"
    )
    manager = models.ForeignKey(
        User,
        limit_choices_to=lambda: {"status": MANAGER},
        related_name="manager_of",
        on_delete=models.CASCADE,
    )

    def __str__(self):
        return self.manager.name + " group"


class AdminUser(User):
    class Meta:
        proxy = True

    def __str__(self):
        return "%s (ADMIN)" % self.username


class Comment(models.Model):
    text = models.TextField()
    author = models.ForeignKey(User, related_name="comments", on_delete=models.CASCADE)

    date = models.DateField()
    time = models.TimeField()

    def __str__(self):
        return "%s said %s" % (self.author, self.text[:25])


class Article(models.Model):
    name = models.CharField(verbose_name="title", max_length=200, blank=True)
    published = models.DateTimeField()
    author = models.ForeignKey(User, null=True, on_delete=models.CASCADE)

    def __str__(self):
        if self.author_id:
            return "%s on %s" % (self.author, self.published)
        return "Anonymous on %s" % self.published


class Book(models.Model):
    title = models.CharField(max_length=100)
    price = models.DecimalField(max_digits=6, decimal_places=2)
    average_rating = models.FloatField()

    def __str__(self):
        return self.title


class Place(models.Model):
    name = models.CharField(max_length=100)

    class Meta:
        abstract = True


class Restaurant(Place):
    serves_pizza = models.BooleanField(default=False)


class NetworkSetting(models.Model):
    ip = models.GenericIPAddressField()
    mask = SubnetMaskField()

    cidr = models.CharField(max_length=18, blank=True, verbose_name="CIDR")


class Company(models.Model):
    name = models.CharField(max_length=100)

    def __str__(self):
        return self.name

    class Meta:
        ordering = ["name"]


class Location(models.Model):
    company = models.ForeignKey(
        Company, related_name="locations", on_delete=models.CASCADE
    )
    name = models.CharField(max_length=100)
    zip_code = models.CharField(max_length=10)
    open_days = models.CharField(max_length=7)

    def __str__(self):
        return "%s: %s" % (self.company.name, self.name)


class Account(models.Model):
    name = models.CharField(max_length=100)
    in_good_standing = models.BooleanField(default=False)
    friendly = models.BooleanField(default=False)


class Profile(models.Model):
    account = models.OneToOneField(
        Account, related_name="profile", on_delete=models.CASCADE
    )
    likes_coffee = models.BooleanField(default=False)
    likes_tea = models.BooleanField(default=False)


class BankAccount(Account):
    amount_saved = models.IntegerField(default=0)


class Node(models.Model):
    name = models.CharField(max_length=20)
    adjacents = models.ManyToManyField("self")


class DirectedNode(models.Model):
    name = models.CharField(max_length=20)
    outbound_nodes = models.ManyToManyField(
        "self", symmetrical=False, related_name="inbound_nodes"
    )


class Worker(models.Model):
    name = models.CharField(max_length=100)


class HiredWorker(models.Model):
    salary = models.IntegerField()
    hired_on = models.DateField()
    worker = models.ForeignKey(Worker, on_delete=models.CASCADE)
    business = models.ForeignKey("Business", on_delete=models.CASCADE)


class Business(models.Model):
    name = models.CharField(max_length=100)
    employees = models.ManyToManyField(
        Worker, through=HiredWorker, related_name="employers"
    )


class UUIDTestModel(models.Model):
    uuid = models.UUIDField()


class SpacewalkRecord(models.Model):
    """Cumulative space walk record.

    See: https://en.wikipedia.org/wiki/List_of_cumulative_spacewalk_records

    """

    astronaut = models.CharField(max_length=100)
    duration = models.DurationField()
